package datos;


import interfaz.dialogos.error.DialogoError;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


import modelo.ficheros.UtilidadesFichero;
import modelo.modeloXML.Ciclo;
import modelo.modeloXML.EntradaFuncion;
import modelo.modeloXML.FicheroAnotado;
import modelo.modeloXML.FicheroInstrumentalizadoInfo;
import modelo.modeloXML.FuncionInfo;
import modelo.modeloXML.LineaAnotada;
import modelo.modeloXML.LineaPerfilPlano;
import modelo.modeloXML.ModeloXML;

public class GprofXMLParser {

	/**
	 * Crea una clase ModeloEstadísticas a partir de un fichero xml (f).
	 * 
	 * @param f
	 * @return
	 * @throws IOException 
	 * @throws SAXException 
	 * @throws ParserConfigurationException 
	 * @throws XPathExpressionException 
	 * @throws NumberFormatException 
	 */
	public ModeloXML leerModeloXML ( File f ) 
	throws ParserConfigurationException, SAXException, 
	IOException, NumberFormatException, XPathExpressionException {

		ModeloXML moXML = new ModeloXML( );


		Document								docXML				= null;

		/** primero se lee el fichero y se añade el contenido a
		 * contenidoFicheroXML como String */
		String contenidoFicheroXML = new UtilidadesFichero ().leerFichero( f );
		moXML.setContenidoFicheroXML ( contenidoFicheroXML );

		/** Ahora se pasa a document el fichero xml para su posterior
		 * interpretación */

		DocumentBuilder docB = DocumentBuilderFactory.newInstance( )
		.newDocumentBuilder( );
		docXML = docB.parse( f );

		/**
		 * ahora se va a leer para rellenar todo lo demás, una vez que tenemos
		 * el docXML
		 */
		if ( docXML != null ) {
			leerGrafoDeLlamadas( moXML, docXML );
			leerFuncionInfo( moXML, docXML );
			leerPerfilPlano( moXML, docXML );
			leerFicheroInfo( moXML, docXML );
			leerFicheroAnotado( moXML, docXML );
			leerSalidaGprof( moXML, docXML );
		}




		return moXML;
	}

	private void leerGrafoDeLlamadas ( ModeloXML mo, Document docXML ) throws NumberFormatException ,
	XPathExpressionException {

		/** primero comprobar que existe el Grafo de Llamadas. */
		String expresion = "/gprofxml/llamadas";
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/**
		 * si existe, rellenar las estructuras correspondientes (EntradaFuncion)
		 */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {

			/**
			 * a continuación, vamos a iniciar el proceso creando las entradas
			 * simples. NOTA: se realiza solo la lectura del principal y no se
			 * añaden los hijos. Eso se hace así porque todavía no están
			 * añadidos los ciclos y no se sabría si añadir como Miembro del
			 * mismo ciclo, o como hijo normal.
			 */
			expresion = "/gprofxml/llamadas/entradasGrafo/entradaSimple";
			NodeList nodosEntradas = (NodeList) xpath.evaluate( expresion ,
					docXML , XPathConstants.NODESET );

			if ( nodosEntradas != null ) {

				/**
				 * realización de la lectura de las entradas simples y creación
				 * de la estructura EntradaFuncion correspondiente.
				 */
				for ( int i = 0 ; i < nodosEntradas.getLength( ) ; i++ ) {
					Node nodoEntradaSimple = nodosEntradas.item( i );
					Node n; // variable auxiliar
					EntradaFuncion entradaFuncion = new EntradaFuncion( );

					/** primero la línea principal de las entradas. */
					Node nodoPrincipal = (Node) xpath.evaluate( "principal" ,
							nodoEntradaSimple , XPathConstants.NODE );

					n = (Node) xpath.evaluate( "indice" , nodoPrincipal ,
							XPathConstants.NODE );
					entradaFuncion.setIndice( Integer.valueOf( n
							.getTextContent( ) ) );
					n = (Node) xpath.evaluate( "nombreFuncion" , nodoPrincipal ,
							XPathConstants.NODE );
					entradaFuncion.setNombreFuncion( n.getTextContent( ) );
					/*n = (Node) xpath.evaluate( "ciclo" , nodoPrincipal ,
							XPathConstants.NODE );
					entradaFuncion.setCiclo( Integer.valueOf( n
							.getTextContent( ) ) );*/
					n = (Node) xpath.evaluate( "tiempoPropioSegundos" ,
							nodoPrincipal , XPathConstants.NODE );
					entradaFuncion.setTiempoPropio( Double.valueOf( n
							.getTextContent( ) ) );
					n = (Node) xpath.evaluate( "tiempoHijosSegundos" ,
							nodoPrincipal , XPathConstants.NODE );
					entradaFuncion.setTiempoHijos( Double.valueOf( n
							.getTextContent( ) ) );
					n = (Node) xpath.evaluate( "espontanea" , nodoPrincipal ,
							XPathConstants.NODE );
					if ( n == null ) {
						n = (Node) xpath.evaluate( "numeroLlamadas" ,
								nodoPrincipal , XPathConstants.NODE );
						entradaFuncion.setNumeroLlamadas( Integer.valueOf( n
								.getTextContent( ) ) );

						n = (Node) xpath.evaluate( "numeroLlamadasRecursivas" ,
								nodoPrincipal , XPathConstants.NODE );

						entradaFuncion
						.setNumeroLlamadasRecursivas( n != null ? Integer
								.valueOf( n.getTextContent( ) )
								: 0 );
					}

					/**
					 * añadir cada EntradaFuncion, con su correspondiente Indice
					 * en un Map para poder recuperarlo.
					 */
					mo.addEntradaFuncion( entradaFuncion);
				}

			}

			/**
			 * ahora, vamos a realizar la lectura de las entradas ciclos y crear
			 * la estructura Ciclo correspondiente.
			 */
			expresion = "/gprofxml/llamadas/entradasGrafo/entradaCiclo";
			NodeList nodosEntradasCiclo = (NodeList) xpath.evaluate( expresion ,
					docXML , XPathConstants.NODESET );

			if ( nodosEntradasCiclo != null ) {
				
				for ( int i = 0 ; i < nodosEntradasCiclo.getLength( ) ; i++ ) {

					Node nodoEntradaCiclo = nodosEntradasCiclo.item( i );
					Node n; // variable auxiliar
					Ciclo ciclo = new Ciclo( );

					/** primero la línea principal del ciclo. */
					Node nodoPrincipalCiclo = (Node) xpath.evaluate(
							"principalCiclo" , nodoEntradaCiclo ,
							XPathConstants.NODE );

					n = (Node) xpath.evaluate( "indice" , nodoPrincipalCiclo ,
							XPathConstants.NODE );
					ciclo.setIndice( Integer.valueOf( n.getTextContent( ) ) );
					
					n = (Node) xpath.evaluate( "ciclo" , nodoPrincipalCiclo ,
							XPathConstants.NODE );
					ciclo.setCiclo( Integer.valueOf( n.getTextContent( ) ) );

					/** a continuación, sus miembros. */
					NodeList nodosMiembro = (NodeList) xpath.evaluate(
							"miembroCiclo" , nodoEntradaCiclo ,
							XPathConstants.NODESET );

					for ( int j = 0 ; j < nodosMiembro.getLength( ) ; j++ ) {
						Node nodoMiembro = nodosMiembro.item( j );

						/**
						 * primero recuperamos la entradaFuncion que corresponde
						 * a este miembro del ciclo.
						 */
						n = (Node) xpath.evaluate( "indice" , nodoMiembro ,
								XPathConstants.NODE );
						EntradaFuncion entradaFuncion = mo.getEntradasFuncion( )
							.get( Integer.valueOf( n.getTextContent( ) ) );

						ciclo.addMiembro( entradaFuncion );
						entradaFuncion.setCiclo( ciclo );
					}

					/**
					 * añadir cada Ciclo, con su correspondiente Indice en un
					 * Map para poder recuperarlo
					 */
					mo.addCiclo( ciclo );
				}

			}

			/**
			 * y por último, los hijos de todas las entradaFuncion. NOTA: Se
			 * realiza después de la lectura de ciclos para que al añadir los
			 * hijos, se sepa si son miembros del mismo ciclo o hijos normales.
			 */
			for ( int i = 0 ; i < nodosEntradas.getLength( ) ; i++ ) {
				Node nodoEntradaSimple = nodosEntradas.item( i );
				Node n; // variable auxiliar

				/**
				 * primero recuperamos la entradaFuncion que corresponde a este
				 * nodoEntradaSimple
				 */
				n = (Node) xpath.evaluate( "principal/indice" ,
						nodoEntradaSimple , XPathConstants.NODE );
				EntradaFuncion entradaFuncion = mo.getEntradasFuncion( ).get( Integer
						.valueOf( n.getTextContent( ) ) );

				/** ahora evaluamos todos sus hijos */
				NodeList nodosHijos = (NodeList) xpath.evaluate( "hijo" ,
						nodoEntradaSimple , XPathConstants.NODESET );

				for ( int j = 0 ; j < nodosHijos.getLength( ) ; j++ ) {
					Node nodoHijo = nodosHijos.item( j );

					/**
					 * primero recuperamos la entradaFuncion que corresponde a
					 * este hijo
					 */
					n = (Node) xpath.evaluate( "indice" , nodoHijo ,
							XPathConstants.NODE );
					EntradaFuncion entradaFuncionHijo = mo.getEntradasFuncion( ).get( Integer
							.valueOf( n.getTextContent( ) ) );

					n = (Node) xpath.evaluate( "numeroLlamadas" , nodoHijo ,
							XPathConstants.NODE );
					Integer numeroLlamadas = Integer.valueOf( n
							.getTextContent( ) );

					if ( entradaFuncion.hasCiclo( )
							&& entradaFuncionHijo.getCiclo( ) == entradaFuncion
							.getCiclo( ) ) {
						entradaFuncion.addHijoMiembroCiclo( entradaFuncionHijo ,
								numeroLlamadas );
					} else {
						entradaFuncion.addHijo( entradaFuncionHijo ,
								numeroLlamadas );
					}

				}
			}

			/** y por último el tiempo total de ejecución */
			expresion = "/gprofxml/llamadas/granularidad/tiempoEjecucion";
			Node nodoPeriodo = (Node) xpath.evaluate( expresion , docXML ,
					XPathConstants.NODE );
			if ( nodoPeriodo != null )
				mo.setTiempoEjecucion (Double
				.valueOf( nodoPeriodo.getTextContent( ) ) );
		}

	}

	private void leerFuncionInfo ( ModeloXML mo, Document docXML )
	throws NumberFormatException ,
	XPathExpressionException {
		/** primero comprobar que existe la información de función. */
		String expresion = "/gprofxml/funcionInfo";
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/** si existe, rellenar las estructuras correspondientes (FuncionInfo) */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {
			/**
			 * a continuación, vamos a iniciar el proceso creando las líneas de
			 * función.
			 */
			expresion = "/gprofxml/funcionInfo/lineaFuncion";
			NodeList nodosLineas = (NodeList) xpath.evaluate( expresion ,
					docXML , XPathConstants.NODESET );

			if ( nodosLineas != null ) {

				/**
				 * realización de la lectura de las líneas de funcion y creación
				 * de la estructura FuncionInfo correspondiente.
				 */
				for ( int i = 0 ; i < nodosLineas.getLength( ) ; i++ ) {
					Node nodoFuncion = nodosLineas.item( i );
					Node n; // variable auxiliar
					FuncionInfo funcionInfo = new FuncionInfo( );

					n = (Node) xpath.evaluate( "nombreFichero" , nodoFuncion ,
							XPathConstants.NODE );
					funcionInfo.setNombreFichero( n.getTextContent( ) );

					n = (Node) xpath.evaluate( "nLinea" , nodoFuncion ,
							XPathConstants.NODE );
					Integer nLinea;
					try {
						nLinea = n.getTextContent( ).matches( "\\d+" ) ? Integer
								.valueOf( n.getTextContent( ) )
								: null;
					} catch ( NumberFormatException e ) {
						nLinea = null;
					}
					funcionInfo.setNumeroLinea( nLinea );

					n = (Node) xpath.evaluate( "nombreFuncion" , nodoFuncion ,
							XPathConstants.NODE );
					funcionInfo.setNombreFuncion( n.getTextContent( ) );

					n = (Node) xpath.evaluate( "dirHex" , nodoFuncion ,
							XPathConstants.NODE );
					String dirHex;
					try {
						dirHex = Integer.toHexString( Integer.decode( n
								.getTextContent( ) ) );
					} catch ( NumberFormatException e ) {
						dirHex = null;
					}
					funcionInfo.setDireccionHexadecimal( dirHex );

					n = (Node) xpath.evaluate( "nEjecuciones" , nodoFuncion ,
							XPathConstants.NODE );
					Integer nEjecuciones;
					try {
						nEjecuciones = n.getTextContent( ).matches( "\\d+" ) ? Integer
								.valueOf( n.getTextContent( ) )
								: null;
					} catch ( NumberFormatException e ) {
						nEjecuciones = null;
					}
					funcionInfo.setNumeroEjecuciones( nEjecuciones );

					mo.addInfoFuncion( funcionInfo );
				}
			}

		}

	}

	private void leerPerfilPlano ( ModeloXML mo, Document docXML ) throws NumberFormatException ,
	XPathExpressionException {
		/** primero comprobar que existe el Perfil Plano. */
		String expresion = "/gprofxml/plano";
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/**
		 * si existe, rellenar las estructuras correspondientes
		 * (LineaPerfilPlano)
		 */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {

			/**
			 * a continuación, vamos a iniciar el proceso creando las líneas de
			 * Perfilado.
			 */
			expresion = "/gprofxml/plano/datosFuncion";
			NodeList nodosLineas = (NodeList) xpath.evaluate( expresion ,
					docXML , XPathConstants.NODESET );

			if ( nodosLineas != null ) {

				/**
				 * realización de la lectura de las líneas del perfil plano y
				 * creación de la estructura LineaPerfilPlano correspondiente.
				 */
				for ( int i = 0 ; i < nodosLineas.getLength( ) ; i++ ) {
					Node nodoLineaPerfilPlano = nodosLineas.item( i );
					Node n; // variable auxiliar
					LineaPerfilPlano lineaPerfilPlano = new LineaPerfilPlano( );

					n = (Node) xpath.evaluate( "nombreFuncion" ,
							nodoLineaPerfilPlano , XPathConstants.NODE );
					lineaPerfilPlano.setNombreFuncion( n.getTextContent( ) );
					n = (Node) xpath.evaluate( "tiempoPropioSegundos" ,
							nodoLineaPerfilPlano , XPathConstants.NODE );
					lineaPerfilPlano.setTiempoPropio( Double.valueOf( n
							.getTextContent( ) ) );
					// datos[i][3] = tiempoTotal += (Double)datos[i][2];
					n = ( (Node) xpath.evaluate( "numeroLlamadas" ,
							nodoLineaPerfilPlano , XPathConstants.NODE ) );
					if ( n != null )
						lineaPerfilPlano.setNumeroLlamadas( Integer.valueOf( n
								.getTextContent( ) ) );
					n = ( (Node) xpath.evaluate(
							"tiempoPorLlamadaTotalMilisegundos" ,
							nodoLineaPerfilPlano , XPathConstants.NODE ) );
					if ( n != null )
						lineaPerfilPlano
						.setEstimacionTiempoTotalPorLlamada( Double
								.valueOf( n.getTextContent( ) ) );

					mo.addLineaPerfilPlano( lineaPerfilPlano );
				}
				// lista calculados
				// for ( int i = 0; i < numeroDeLineas; i++ ) {
				// datos[i][1] = (Double)datos[i][2] / tiempoTotal * 100;
				// }
			}
			/** y por último el periodo de la muestra */
			expresion = "/gprofxml/plano/tiempoPeriodo";
			Node nodoPeriodo = (Node) xpath.evaluate( expresion , docXML ,
					XPathConstants.NODE );
			if ( nodoPeriodo != null )
				mo.setPeriodoMuestra( Double.valueOf( nodoPeriodo.getTextContent( ) ) );
		}
	}

	private void leerFicheroInfo ( ModeloXML mo, Document docXML ) throws NumberFormatException ,
	XPathExpressionException {
		/**
		 * primero comprobar que existe la información del fichero de
		 * instrumentalización.
		 */
		String expresion = "/gprofxml/perfilado";
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/**
		 * si existe, rellenar las estructuras correspondientes
		 * (FicheroIntrumentalizadoInfo)
		 */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {
			/**
			 * a continuación, vamos a iniciar el proceso leyendo las líneas de
			 * función.
			 */
			expresion = "/gprofxml/perfilado/lineaPerfilado";
			NodeList nodosLineas = (NodeList) xpath.evaluate( expresion ,
					docXML , XPathConstants.NODESET );

			if ( nodosLineas != null ) {
				/**
				 * realización de la lectura de las líneas de funcion y creación
				 * de la estructura FuncionInfo correspondiente.
				 */
				for ( int i = 0 ; i < nodosLineas.getLength( ) ; i++ ) {
					Node nodoLineaFicheroInfo = nodosLineas.item( i );
					Node n; // variable auxiliar
					FicheroInstrumentalizadoInfo fichero = new FicheroInstrumentalizadoInfo( );

					n = (Node) xpath.evaluate( "nombreFichero" ,
							nodoLineaFicheroInfo , XPathConstants.NODE );
					fichero.setNombreFichero( n.getTextContent( ) );

					n = (Node) xpath.evaluate( "histogramas" ,
							nodoLineaFicheroInfo , XPathConstants.NODE );
					Integer histogramas;
					try {
						histogramas = n != null? n.getTextContent( ).matches( "\\d+" ) ? Integer
								.valueOf( n.getTextContent( ) )
								: null : null;
					} catch ( NumberFormatException e ) {
						histogramas = null;
					}
					fichero.setHistogramas( histogramas );

					n = (Node) xpath.evaluate( "grafos" , nodoLineaFicheroInfo ,
							XPathConstants.NODE );
					Integer grafos;
					try {
						grafos = n != null? n.getTextContent( ).matches( "\\d+" ) ? Integer
								.valueOf( n.getTextContent( ) )
								: null : null;
					} catch ( NumberFormatException e ) {
						grafos = null;
					}
					fichero.setGrafos( grafos );

					n = (Node) xpath.evaluate( "bloques" ,
							nodoLineaFicheroInfo , XPathConstants.NODE );
					Integer bloques;
					try {
						bloques =  n != null? n.getTextContent( ).matches( "\\d+" ) ? Integer
								.valueOf( n.getTextContent( ) )
								: null : null;
					} catch ( NumberFormatException e ) {
						bloques = null;
					}
					fichero.setBloques( bloques );

					mo.addInfoFichero( fichero );
				}
			}

		}

	}

	private void leerFicheroAnotado ( ModeloXML mo, Document docXML ) throws XPathExpressionException {

		/** primero comprobar que existe el Perfil Plano. */
		String expresion = "/gprofxml/anotado";
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/**
		 * si existe, rellenar las estructuras correspondientes
		 * (LineaPerfilPlano)
		 */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {

			/**
			 * a continuación, vamos a iniciar el proceso creando los ficheros
			 * anotados.
			 */

			String expression = "/gprofxml/anotado/fichero";
			NodeList nodosFicheros = (NodeList) xpath.evaluate( expression ,
					docXML , XPathConstants.NODESET );

			/** Creamos todos los ficheros anotados. */
			for ( int i = 0 ; i < nodosFicheros.getLength( ) ; i++ ) {
				Node nodoFicheroAnotado = nodosFicheros.item( i );
				Node n; // variable auxiliar

				FicheroAnotado fichero = new FicheroAnotado( );

				/** Primero el nombre del fichero. */
				n = (Node) xpath.evaluate( "nombreFichero" ,
						nodoFicheroAnotado , XPathConstants.NODE );
				fichero.setNombreFichero( n.getTextContent( ) );

				/** Después todas sus líneas. */
				expression = "/gprofxml/anotado/fichero[nombreFichero='"
					+ fichero.getNombreFichero( ) + "']/lineaAnotada";
				NodeList nodosLineas = (NodeList) xpath.evaluate( expression ,
						docXML , XPathConstants.NODESET );

				Node datosLinea;
				for ( int j = 0 ; j < nodosLineas.getLength( ) ; j++ ) {
					datosLinea = nodosLineas.item( j );

					LineaAnotada l = new LineaAnotada( );

					n = (Node) xpath.evaluate( "nLinea" , datosLinea ,
							XPathConstants.NODE );
					Integer nlinea;
					try {
						nlinea = n.getTextContent( ).matches( "\\d+" ) ? Integer
								.valueOf( n.getTextContent( ) )
								: null;
					} catch ( NumberFormatException e ) {
						nlinea = null;
					}
					l.setNumeroLinea( nlinea );

					n = ( (Node) xpath.evaluate( "nVeces" , datosLinea ,
							XPathConstants.NODE ) );
					if ( n != null ) {
						Integer nveces;
						try {
							nveces = Integer.valueOf( n.getTextContent( ) );
						} catch ( NumberFormatException e ) {
							nveces = null;
						}
						l.setEjecuciones( nveces );
					}

					n = (Node) xpath.evaluate( "lineaOriginal" , datosLinea ,
							XPathConstants.NODE );
					l.setLineaOriginal( n.getTextContent( ) );

					fichero.addLineaAnotada( l );
				}

				/** Por último, recogeremos el top de las líneas. */
				expression = "/gprofxml/anotado/fichero[nombreFichero='"
					+ fichero.getNombreFichero( ) + "']/top/lineaTop";
				NodeList nodosTop = (NodeList) xpath.evaluate( expression ,
						docXML , XPathConstants.NODESET );

				Node datosTop;
				for ( int j = 0 ; j < nodosTop.getLength( ) ; j++ ) {
					datosTop = nodosTop.item( j );

					n = (Node) xpath.evaluate( "nLinea" , datosTop ,
							XPathConstants.NODE );
					Integer nlinea;
					try {
						nlinea = n.getTextContent( ).matches( "\\d+" ) ? Integer
								.valueOf( n.getTextContent( ) )
								: null;
					} catch ( NumberFormatException e ) {
						nlinea = null;
					}

					fichero.addTopLinea( fichero.getLineasAnotadas( ).get(
							nlinea ) );
				}

				/** Y para terminar, el resumen. */
				expression = "/gprofxml/anotado/fichero[nombreFichero='"
					+ fichero.getNombreFichero( ) + "']/resumen";
				Node nodoResumen = (Node) xpath.evaluate( expression , docXML ,
						XPathConstants.NODE );

				n = (Node) xpath.evaluate( "lineasEjecutables" , nodoResumen ,
						XPathConstants.NODE );
				Integer nejecutables;
				try {
					nejecutables = n.getTextContent( ).matches( "\\d+" ) ? Integer
							.valueOf( n.getTextContent( ) )
							: null;
				} catch ( NumberFormatException e ) {
					nejecutables = null;
				}
				fichero.setTotalLineasEjecutables( nejecutables );

				n = (Node) xpath.evaluate( "lineasEjecutadas" , nodoResumen ,
						XPathConstants.NODE );
				Integer nejecutadas;
				try {
					nejecutadas = n.getTextContent( ).matches( "\\d+" ) ? Integer
							.valueOf( n.getTextContent( ) )
							: null;
				} catch ( NumberFormatException e ) {
					nejecutadas = null;
				}
				fichero.setTotalLineasEjecutables( nejecutadas );

				n = (Node) xpath.evaluate( "totalEjecuciones" , nodoResumen ,
						XPathConstants.NODE );
				Integer tejecuciones;
				try {
					tejecuciones = n.getTextContent( ).matches( "\\d+" ) ? Integer
							.valueOf( n.getTextContent( ) )
							: null;
				} catch ( NumberFormatException e ) {
					tejecuciones = null;
				}
				fichero.setTotalLineasEjecutables( tejecuciones );

				mo.addFicheroAnotado( fichero );

			}

		}

	}

	private void leerSalidaGprof ( ModeloXML mo, Document docXML )
	throws XPathExpressionException {
		String s = new String( );

		// leer la salida gprof del xml:

		// 1º LEER DEL XML y construir String con la salida estándar.
		XPath xpath = XPathFactory.newInstance( ).newXPath( );
		String expression = "/gprofxml/salida";
		Node nodoSalida = null;

		// Esto puede dar problemas de lectura de la salida gprof debido a que
		// el throw XPathException lo lanzo y no lo try/catch aquí... :S
		nodoSalida = (Node) xpath.evaluate( expression , docXML ,
				XPathConstants.NODE );

		if ( nodoSalida == null ) {
			s = "";
		} else {
			s = nodoSalida.getTextContent( );
		}

		mo.setSalidaGprof ( s );
	}


	public static void main ( String [] args ) {
		javax.swing.JFileChooser f = new javax.swing.JFileChooser( );

		if ( f.showOpenDialog( new javax.swing.JFrame( ) ) == javax.swing.JFileChooser.APPROVE_OPTION ) {
			ModeloXML mo;
			try {
				mo = new GprofXMLParser( ).leerModeloXML( f.getSelectedFile( ) );

				Iterator < Ciclo > i = mo.getCiclos( ).values( ).iterator( );
				while ( i.hasNext( ) ) {
					System.out.print( i.next( ).toString( ) );
				}

				Iterator < EntradaFuncion > j = mo.getEntradasFuncion( ).values( )
				.iterator( );
				while ( j.hasNext( ) ) {
					System.out.print( j.next( ).toString( ) + "\n" );
				}

				Iterator < LineaPerfilPlano > k = mo.getPerfilPlano( ).iterator( );
				while ( k.hasNext( ) ) {
					System.out.print( k.next( ).toString( ) + "\n" );
				}

				Iterator < FicheroAnotado > l = mo.getFicherosAnotados( ).values( )
				.iterator( );
				while ( l.hasNext( ) ) {
					System.out.println( l.next( ).toString( ) + "\n" );
				}

				Iterator < FuncionInfo > m = mo.getInfoFunciones( ).iterator( );
				while ( m.hasNext( ) ) {
					System.out.println( m.next( ).toString( ) + "\n" );
				}

				System.out.println( mo.getInfoFichero( ).toString( ) + "\n" );
			} catch ( Exception e ) {
				java.util.ResourceBundle		errores				= 
					java.util.ResourceBundle.getBundle( "interfaz.errores.Errores" );

				DialogoError.getInstance( ).showErrorDialog( errores.getString( "err_errorLecturaXML" ) );
				e.printStackTrace( );
			}
		}

	}
}
