/*
 * Clase GUI.java es una aplicación Java.
 * Pertenece a la aplicación guigprof para el proyecto fin de carrera de 
 * Ángel García Fernández.
 * Tutor: Francisco Palomo Lozano e Inmaculada Medina Bulo
 * Fecha: Julio de 2008.
 */

package interfaz;

import modelo.Sesion;
import modelo.ficheros.TipoFichero;
import modelo.ficheros.UtilidadesExtensionFichero;
import modelo.vcompilacion.ModeloCompilacion;
import modelo.vedicion.ModeloEdicion;
import modelo.vejecucion.ModeloEjecucion;
import modelo.vestadisticas.ModeloEstadistica;
import modelo.vgprof.ModeloGprof;
import modelo.vtraduccion.ModeloTR;
import negocio.PreferenciasSingleton;
import negocio.guigprof;
import interfaz.acciones.*;
import interfaz.dialogos.acercaDe.DialogoAcercaDe;
import interfaz.dialogos.dialogoAbrir.DialogoAbrir;
import interfaz.dialogos.dialogoPreferencias.DialogoPreferencias;
import interfaz.dialogos.dialogoPreferencias.EstadoDialogoPreferencias;
import interfaz.dialogos.dialogoSalir.DialogoCerrarWindowListener;
import interfaz.dialogos.dialogoSesion.DialogoSesion;
import interfaz.dialogos.error.DialogoError;
import interfaz.vcompilacion.VistaCompilacion;
import interfaz.vedicion.VistaEdicion;
import interfaz.vejecucion.VistaEjecucion;
import interfaz.vestadisticas.VistaEstadisticas;
import interfaz.vgprof.VistaGprof;
import interfaz.vtraductor.VistaTR;

//eventos
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.io.File; //import java.awt.event.ActionListener;

import java.util.Iterator;
import java.util.ResourceBundle; //import java.io.IOException;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.border.EtchedBorder;
//Layout
import net.miginfocom.swing.MigLayout;

/**
 * Esta clase crea la ventana principal y es la clase principal del GUI. La ventana está
 * compuesta por un menú y varias vistas, cada una con varias pestañas.
 * 
 * @author Ángel García Fernández
 * @version 0.1.0
 * @since 0.1
 */
public class GUI extends JFrame implements InterfazAcciones
	{
	/** serial Version UID */
	final static long				serialVersionUID	= 1L;

	/** recursos para dar soporte sobre idiomas	 */
	static ResourceBundle					idiomas				= ResourceBundle
																.getBundle( "interfaz.idiomas.Recursos" );
	/** recursos de iconos etc...	 */
	static ResourceBundle					recursos			= ResourceBundle
																.getBundle( "interfaz.Recursos" );
	/** Recursos para dar soporte de idiomas a los errores. */
	static ResourceBundle					errores				= ResourceBundle
		.getBundle( "interfaz.errores.Errores" );

	static String							TITULO_VENTANA		= "GUI gprof";
	
	
	/** Acciones */
	{
		CrearYGestionarAcciones.createInstance ( this );
	}
	CrearYGestionarAcciones acciones = 	CrearYGestionarAcciones.getInstance( );
	
	/** Vistas */
	VistaEdicion					edicion = new VistaEdicion(this);
	VistaCompilacion				compilacion = new VistaCompilacion(this);
	VistaEjecucion					ejecucion = new VistaEjecucion(this);
	VistaGprof						gprof = new VistaGprof(this);
	VistaTR							traductor = new VistaTR(this);
	VistaEstadisticas				estadisticas = new VistaEstadisticas(this);
	
	Vector <Vista>	vistas = new Vector <Vista> ();
	{
		vistas.add( edicion );
		vistas.add( compilacion );
		vistas.add( ejecucion );
		vistas.add( gprof );
		vistas.add( traductor );
		vistas.add( estadisticas );
	}
	
	/** dialogo Abrir */ 
	DialogoAbrir					dialogoAbrir		= new DialogoAbrir( );
	
	/** dialogo de Preferencias */
	DialogoPreferencias				dialogoPreferencias	 = new DialogoPreferencias( this , 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ) );
	
	JFrame this_frame = this;
	

	private JMenuItem	accionAyuda;
	{/**
	 * Es la acción de Ayuda del
	 * programa.  acción de aquí. 
	 */
		accionAyuda = new JMenuItem ( idiomas.getString( "eti_ayuda" ) );
		accionAyuda.setToolTipText( idiomas.getString( "tip_ayuda" ) );
		accionAyuda.setAccelerator( (KeyStroke) idiomas.getObject( "key_ayuda" ) );
		accionAyuda.setMnemonic( ( (Integer) idiomas.getObject( "mne_ayuda" ) ).intValue() );

		try {
			guigprof.getInstance().enlazarAyuda( accionAyuda, this.getContentPane( ) );

		} catch ( Exception e ) {
			//JOptionPane.showMessageDialog( null , errores.getString( "err_ayuda" ) );
			DialogoError.getInstance( ).showErrorDialog( errores.getString( "err_ayuda" ) );
			e.printStackTrace( );
		}
	}

	/** Fin- Declaración Acciones */


	/**
	 * Constructor de la clase que crea la ventana y todo los componentes
	 */
	public GUI (  ) {

		// crear ventana principal
		super( TITULO_VENTANA );

		this.addWindowListener( new DialogoCerrarWindowListener( this ) );

		this.setIconImage( ( (ImageIcon) recursos.getObject( "icn_Aplicacion" ) ).getImage() ) ;
		
		//this.acciones = new CreadorAcciones ( this );
		this.crearComponentes( );
		this.crearMenu( );

		/** No hace nada para que salga el diálogo para cerrar. */
		this.setDefaultCloseOperation( javax.swing.JFrame.DO_NOTHING_ON_CLOSE );
		/** Asignaremos un tamaño de referencia en pixels. */
		this.setPreferredSize( new Dimension( 800 , 600 ) );
		/** Recordar empaquetar ventana. */
		this.pack( );
		/** Asegura que la aplicación empieza maximizada. */
		this.setExtendedState( JFrame.MAXIMIZED_BOTH );
		/** Mostrar ventana. */
		this.setVisible( true );
	}

	/*
	 * protected void cargarPreferencias () { //elegir la carpeta de salida para
	 * guardar u abrir... cargarlo en dialogoAbrir }
	 */// No hace falta, ahora lo hace la clase guigprof...

	// menú
	JMenuBar			barraDeMenu	= new JMenuBar( );

	private JToolBar	barraDeVistas;

	private JToolBar	barraDeTareasEdicion;

	private JToolBar	barraDeTareasEjecucion;

	private JToolBar	barraDeTareasCompilacion;
	
	private JToolBar	barraDeTareasGprof;

	private JToolBar	barraDeTareasEstadisticas;
	
	private JToolBar	barraDeTareasTR;

	protected void crearBarraVistas ( ) {

		JButton botVistaEdicion = new JButton( );
		{
			botVistaEdicion.setHideActionText( true );
			botVistaEdicion.setAction( acciones.getAccionVistaEdicion( ) );
		}

		JButton botVistaCompilacion = new JButton( );
		{
			botVistaCompilacion.setHideActionText( true );
			botVistaCompilacion.setAction( acciones.getAccionVistaCompilacion( ) );
		}

		JButton botVistaEjecucion = new JButton( );
		{
			botVistaEjecucion.setHideActionText( true );
			botVistaEjecucion.setAction( acciones.getAccionVistaEjecucion( ) );
		}
		
		JButton botVistaGprof = new JButton( );
		{
			botVistaGprof.setHideActionText( true );
			botVistaGprof.setAction( acciones.getAccionVistaGprof( ) );
		}
		
		JButton botVistaTR = new JButton( );
		{
			botVistaTR.setHideActionText( true );
			botVistaTR.setAction( acciones.getAccionVistaTR( ) );
		}
		
		JButton botVistaEstadisticas = new JButton( );
		{
			botVistaEstadisticas.setHideActionText( true );
			botVistaEstadisticas.setAction( acciones.getAccionVistaEstadisticas( ) );
		}
		// Barra de tareas de vistas:
		// JToolBar barraDeVistas;
		{
			barraDeVistas = new JToolBar( );

			barraDeVistas.setAutoscrolls( true );
			barraDeVistas.setFloatable( false );
			barraDeVistas.add( botVistaEdicion );
			barraDeVistas.add( botVistaCompilacion );
			barraDeVistas.add( botVistaEjecucion );
			barraDeVistas.add( botVistaGprof );
			barraDeVistas.add( botVistaTR );
			barraDeVistas.add( botVistaEstadisticas );
		}
	}

	protected void crearBarraTareas ( ) {
		barraDeTareasEdicion = edicion.getBarraDeTareas( );

		barraDeTareasEjecucion = ejecucion.getBarraDeTareas( );

		barraDeTareasCompilacion = compilacion.getBarraDeTareas( );

		barraDeTareasGprof = gprof.getBarraDeTareas( );
		
		barraDeTareasEstadisticas = estadisticas.getBarraDeTareas( );
		
		barraDeTareasTR = traductor.getBarraDeTareas( );
	}

	protected void crearMenu ( ) {
		/** menú Archivo */
		JMenu menuArchivo = new JMenu( new AbstractAction( idiomas
				.getString( "eti_men_archivo" ) ) {
			private static final long	serialVersionUID	= 1L;

			{
				putValue( MNEMONIC_KEY , idiomas.getObject( "mne_men_archivo" ) );
			}

			public void actionPerformed ( ActionEvent e ) {
			}
		} );

		JMenu subMenuAbrir = new JMenu( idiomas.getString( "eti_men_abrir" ) );
		subMenuAbrir.setMnemonic( ( (Integer) idiomas.getObject( "mne_men_abrir" ) ).intValue() );
		
		subMenuAbrir.add( acciones.getAccionAbrirFuente( ) );
		subMenuAbrir.add( acciones.getAccionCrearCompilacion ( ) );
		subMenuAbrir.add( acciones.getAccionAbrirEjecucion ( ) );
		subMenuAbrir.add( acciones.getAccionAbrirDatosEjecucion ( ) );
		subMenuAbrir.add( acciones.getAccionGenerarXML ( ) );
		subMenuAbrir.add( acciones.getAccionAbrirEstadisticasXML ( ) );
		menuArchivo.add( subMenuAbrir );
		menuArchivo.add( acciones.getAccionAbrirSesion ( ) );		
		menuArchivo.addSeparator( );
		menuArchivo.add( acciones.getAccionCerrar ( ) );
		menuArchivo.add( acciones.getAccionCerrarTodo ( ) );
		menuArchivo.addSeparator( );
		menuArchivo.add( acciones.getAccionGuardarSesion ( ) );
		menuArchivo.addSeparator();
		menuArchivo.add( acciones.getAccionOpcionesAplicacion ( ) );		
		menuArchivo.addSeparator( );
		menuArchivo.add( acciones.getAccionSalir ( ) );

		barraDeMenu.add( menuArchivo );

		/** menú Edicion */
		JMenu menuEdicion = new JMenu( new AbstractAction( idiomas
				.getString( "eti_men_edicion" ) ) {
			private static final long	serialVersionUID	= 1L;

			{
				putValue( MNEMONIC_KEY , idiomas.getObject( "mne_men_edicion" ) );
			}

			public void actionPerformed ( ActionEvent e ) {
			}
		} );

		menuEdicion.add( acciones.getAccionAbrirFuente ( ) );
		menuEdicion.addSeparator( );
		menuEdicion.add( acciones.getAccionEditar ( ) );
		menuEdicion.add( acciones.getAccionActualizar ( ) );
		menuEdicion.addSeparator( );
		menuEdicion.add( acciones.getAccionCompilarFuente ( ) );
		

		barraDeMenu.add( menuEdicion );

		/**  menú Compilacion */
		JMenu menuCompilacion = new JMenu( new AbstractAction( idiomas
				.getString( "eti_men_compilacion" ) ) {
			private static final long	serialVersionUID	= 1L;

			{
				putValue( MNEMONIC_KEY , idiomas
						.getObject( "mne_men_compilacion" ) );
			}

			public void actionPerformed ( ActionEvent e ) {
			}
		} );

		menuCompilacion.add( acciones.getAccionCrearCompilacion ( ) );
		menuCompilacion.addSeparator( );
		menuCompilacion.add( acciones.getAccionCompilar ( ) );
		menuCompilacion.add( acciones.getAccionAñadirFicheros ( ) );
		menuCompilacion.add( acciones.getAccionQuitarFicheros ( ) );
		menuCompilacion.add( acciones.getAccionEditarComandoCompilacion ( ) );
		menuCompilacion.add( acciones.getAccionCambiarDirectorioTrabajoCompilacion ( ) );
		menuCompilacion.add( acciones.getAccionNombreEjecutable ( ) );
		menuCompilacion.addSeparator( );
		menuCompilacion.add( acciones.getAccionEjecutarCompilado ( ) );

		barraDeMenu.add( menuCompilacion );

		/** menuEjecucion */
		JMenu menuEjecucion = new JMenu( new AbstractAction( idiomas
				.getString( "eti_men_ejecucion" ) ) {
			private static final long	serialVersionUID	= 1L;

			{
				putValue( MNEMONIC_KEY , idiomas
						.getObject( "mne_men_ejecucion" ) );
			}

			public void actionPerformed ( ActionEvent e ) {
			}
		} );

		menuEjecucion.add( acciones.getAccionAbrirEjecucion ( ) );
		menuEjecucion.addSeparator( );
		menuEjecucion.add( acciones.getAccionEjecutar ( ) );
		menuEjecucion.add( acciones.getAccionAñadirEntradas ( ) );
		menuEjecucion.add( acciones.getAccionQuitarEntradas ( ) );
		menuEjecucion.add( acciones.getAccionAñadirFicheroEntradas ( ) );
		menuEjecucion.add( acciones.getAccionAñadirArgumentos ( ) );
		menuEjecucion.add( acciones.getAccionQuitarArgumentos ( ) );
		menuEjecucion.add( acciones.getAccionCambiarDirectorioTrabajoEjecucion ( ) );
		//menuEjecucion.add( acciones.getAccionSumarResultados ( ) );
		menuEjecucion.add( acciones.getAccionNombreGmonOut ( ) );
		menuEjecucion.addSeparator( );
		menuEjecucion.add( acciones.getAccionUsarGprofConEjecucion ( ) );


		barraDeMenu.add( menuEjecucion );

		/** menúGprof */
		JMenu menuGprof	= new JMenu( new AbstractAction( idiomas
				.getString( "eti_men_gprof" ) ) {
			private static final long	serialVersionUID	= 1L;

			{
				putValue( MNEMONIC_KEY , idiomas
						.getObject( "mne_men_gprof" ) );
			}

			public void actionPerformed ( ActionEvent e ) {
			}
		} );
		
		menuGprof.add( acciones.getAccionAbrirDatosEjecucion ( ) );
		menuGprof.addSeparator( );
		menuGprof.add( acciones.getAccionGprof ( ) );
		menuGprof.add( acciones.getAccionEditarProgramaGprof ( ) );
		menuGprof.add( acciones.getAccionEditarDatosEjecucionGprof ( ) );
		menuGprof.add(	acciones.getAccionEditarComandoGprof ( ) );
		menuGprof.add( acciones.getAccionCambiarDirectorioTrabajoGprof ( ) );
		menuGprof.add( acciones.getAccionNombreSalida ( ) );
		menuGprof.addSeparator( );
		menuGprof.add( acciones.getAccionPasarAXML ( ) );
		
		barraDeMenu.add( menuGprof );
		
		/** menúTR */
		JMenu menuTR	= new JMenu( new AbstractAction( idiomas
				.getString( "eti_men_TR" ) ) {
			private static final long	serialVersionUID	= 1L;

			{
				putValue( MNEMONIC_KEY , idiomas
						.getObject( "mne_men_TR" ) );
			}

			public void actionPerformed ( ActionEvent e ) {
			}
		} );
		
		menuTR.add( acciones.getAccionGenerarXML ( ) );
		menuTR.addSeparator( );
		menuTR.add( acciones.getAccionTraducir ( ) );
		menuTR.add( acciones.getAccionEditarSalidaGprof ( ) );
		menuTR.add(	acciones.getAccionEditarComandoTR ( ) );
		menuTR.add( acciones.getAccionCambiarDirectorioTrabajoTR ( ) );
		menuTR.add( acciones.getAccionNombreXML( ) );
		menuTR.addSeparator( );
		menuTR.add( acciones.getAccionVerEstadisticas ( ) );
		
		barraDeMenu.add( menuTR );
		
		/** menuEstadisticas */
		JMenu menuEstadisticas = new JMenu( new AbstractAction( idiomas
				.getString( "eti_men_estadisticas" ) ) {
			private static final long	serialVersionUID	= 1L;

			{
				putValue( MNEMONIC_KEY , idiomas
						.getObject( "mne_men_estadisticas" ) );
			}

			public void actionPerformed ( ActionEvent e ) {
			}
		} );

		menuEstadisticas.add( acciones.getAccionAbrirEstadisticasXML ( ) );
		menuEstadisticas.addSeparator( );
		menuEstadisticas.add( acciones.getAccionVerPerfilPlano( ) );
		menuEstadisticas.add( acciones.getAccionVerGrafoLlamadas ( ) );
		menuEstadisticas.add( acciones.getAccionVerFuenteAnotado ( ) );
		menuEstadisticas.add( acciones.getAccionVerSalida ( ) );
		menuEstadisticas.addSeparator( );
		menuEstadisticas.add( acciones.getAccionEditarXML ( ) );

		barraDeMenu.add( menuEstadisticas );

		/** menuVentana */
		JMenu menuVentana = new JMenu( new AbstractAction( idiomas
				.getString( "eti_men_ventana" ) ) {
			private static final long	serialVersionUID	= 1L;

			{
				putValue( MNEMONIC_KEY , idiomas.getObject( "mne_men_ventana" ) );
			}

			public void actionPerformed ( ActionEvent e ) {
			}
		} );

		menuVentana.add( acciones.getAccionVistaEdicion ( ) );
		menuVentana.add( acciones.getAccionVistaCompilacion ( ) );
		menuVentana.add( acciones.getAccionVistaEjecucion ( ) );
		menuVentana.add( acciones.getAccionVistaGprof ( ) );
		menuVentana.add( acciones.getAccionVistaTR ( ) );
		menuVentana.add( acciones.getAccionVistaEstadisticas ( ) );

		barraDeMenu.add( menuVentana );

		/** menuAyuda */
		JMenu menuAyuda = new JMenu( new AbstractAction( idiomas
				.getString( "eti_men_ayuda" ) ) {
			private static final long	serialVersionUID	= 1L;

			{
				putValue( MNEMONIC_KEY , idiomas.getObject( "mne_men_ayuda" ) );
			}

			public void actionPerformed ( ActionEvent e ) {
			}
		} );

		menuAyuda.add( accionAyuda );
		menuAyuda.add( acciones.getAccionAcercaDe ( ) );

		barraDeMenu.add( menuAyuda );
	}

	protected void crearComponentes ( ) {

		/**  Panel completo */
		JPanel panelDeContenido = new JPanel( new MigLayout(
				"insets 10px, hidemode 3," ,/* debug, ", */// Condiciones del
															// Layout
				// "[grow, push] 20px [min!]", //Condiciones de las columnas
				// "[min!] 20px [grow, push]")); //Condiciones de las filas
				"[grow] 20px [min!]" , // Condiciones de las columnas
				"[min!] 20px [grow]" ) ); // Condiciones de las filas

		panelDeContenido.setBorder( new EtchedBorder( ) );
		panelDeContenido.setOpaque( true );

		crearBarraVistas( );
		crearBarraTareas( );

		{/**  crear todas las Barras de herramientas: */
			panelDeContenido.add( barraDeVistas , "cell 1 0" );

			panelDeContenido.add( barraDeTareasEdicion , "cell 0 0, grow" );
			panelDeContenido.add( barraDeTareasCompilacion , "cell 0 0, grow" );
			panelDeContenido.add( barraDeTareasEjecucion , "cell 0 0,grow" );
			panelDeContenido.add( barraDeTareasGprof, "cell 0 0,grow" );
			panelDeContenido.add( barraDeTareasTR, "cell 0 0, grow" );
			panelDeContenido.add( barraDeTareasEstadisticas , "cell 0 0,grow" );
		}

		{/** crear panel edicion */
			panelDeContenido.add( edicion , "cell 0 1 2 1, grow" );
			edicion.setActivo( false );
		}

		{/** crear panel compilación */
			compilacion.setActivo( false );

			panelDeContenido.add( compilacion , "cell 0 1 2 1, grow" );
		}

		{/** crear panel gprof */
			gprof.setActivo( false );

			panelDeContenido.add( gprof , "cell 0 1 2 1, grow" );
		}
		
		{/** crear panel ejecución */
			ejecucion.setActivo( false );

			panelDeContenido.add( ejecucion , "cell 0 1 2 1, grow" );
		}

		{/** crear panel traducción */
			traductor.setActivo( false );

			panelDeContenido.add( traductor , "cell 0 1 2 1, grow" );
		}
		
		{/** crear panel estadísticas */
			estadisticas.setActivo( false );

			panelDeContenido.add( estadisticas , "cell 0 1 2 1, grow" );
		}

		// crear menú
		this.setJMenuBar( barraDeMenu );

		setContentPane( panelDeContenido );
	}



	/******************** métodos *****************/

		
	public void cambiarFuente( Font f ) {
			edicion.cambiarFuente( f );
			gprof.cambiarFuente( f );
			traductor.cambiarFuente( f );
	}


	/***************************************/
	
	
	public boolean isVacia ( ) {
		
		return ( edicion != null &&
				compilacion != null &&
				ejecucion != null &&
				gprof != null &&
				traductor != null &&
				estadisticas != null &&
				edicion.getModelos( ).isEmpty( ) &&
				compilacion.getModelos( ).isEmpty( ) &&
				ejecucion.getModelos( ).isEmpty( ) &&
				gprof.getModelos( ).isEmpty( ) &&
				traductor.getModelos( ).isEmpty( ) &&
				estadisticas.getModelos( ).isEmpty( ) );
	}
	
	public boolean isNonVista ( ) {
		
		return ( !edicion.isVisible( ) &&
				!compilacion.isVisible( ) &&
				!ejecucion.isVisible( ) &&
				!gprof.isVisible( ) &&
				!traductor.isVisible( ) &&
				!estadisticas.isVisible( ) );
	}

	public Vista getVistaActiva( ) {
		Vista activa = null;
		if ( edicion.isActivo( ) ) {
			activa = edicion;
		} else if ( compilacion.isActivo( ) ) {
			activa = compilacion;
		} else if ( ejecucion.isActivo( ) ) {
			activa = ejecucion;
		} else if ( gprof.isActivo( ) ) {
			activa = gprof;
		} else if ( estadisticas.isActivo( ) ) {
			activa = estadisticas;
		} else if ( traductor.isActivo( ) ) {
			activa = traductor;
		}
		
		return activa;
	}
	
	@Override
	public void abrirEdicion ( ) {
		this.setVistaActiva( edicion );
		
		edicion.abrirEdicion();
		
	}

	@Override
	public void cerrarPestañaActiva ( ) {
		Vista v = getVistaActiva();
		
		if ( v != null ) {
			v.cerrarPestañaActiva();
		}
		
	}
	
	public void setVistaActiva( Vista v ) {
		if ( !v.isActivo( ) ) {
			Iterator <Vista> it = vistas.iterator( );
			while (it.hasNext( )) { 
				it.next( ).setActivo( false );
			}
			v.setActivo( true );
		}
	}

	@Override
	public void vistaActivaEdicion ( ) {
		setVistaActiva(edicion);
	}

	@Override
	public void vistaActivaEjecucion ( ) {
		setVistaActiva(ejecucion);	
	}

	@Override
	public void vistaActivaEstadisticas ( ) {
		setVistaActiva(estadisticas);
	}

	@Override
	public void vistaActivaTR ( ) {
		setVistaActiva(traductor);
	}

	@Override
	public void vistaActivaGprof ( ) {
		setVistaActiva(gprof);
	}

	@Override
	public void vistaActivaCompilacion ( ) {
		setVistaActiva(compilacion);
	}

	
	@Override
	public void cerrarTodo() {
		if ( JOptionPane.showConfirmDialog( guigprof.getInstance( ).getGUI( ) , 
				idiomas.getString( "q_cerrarTodo_seguroCerrar" ) , 
				GUI.TITULO_VENTANA , 
				JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION ) { 
					Iterator <Vista> it = vistas.iterator( );
					while ( it.hasNext( ) ) 
						it.next().cerrarTodo();
		}
	}

	@Override
	public void editar ( ) {
		this.setVistaActiva( edicion );
		edicion.editar();		
	}

	@Override
	public void crearCompilacion ( ) {
		this.setVistaActiva( compilacion );
		compilacion.crearCompilacion( );
	}

	@Override
	public void abrirPrograma ( ) {
		this.setVistaActiva( ejecucion );
		ejecucion.abrirPrograma();
		
	}

	@Override
	public void abrirDatosEjecucion ( ) {
		this.setVistaActiva( gprof );
		gprof.abrirDatosEjecucion();
	}

	@Override
	public void abrirEstadisticasXML ( ) {
		this.setVistaActiva( estadisticas );
		estadisticas.abrirEstadisticasXML( );		
	}

	@Override
	public void salir ( ) {
		/** Lanza el evento cerrar que lanza la pregunta como
		 * si cerrara la ventana directamente desde la x. */
		this.dispatchEvent(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));		
	}

	@Override
	public void actualizar ( ) {
		this.setVistaActiva( edicion );
		edicion.actualizar( );		
	}

	@Override
	public void compilarFuente ( ) {
		setVistaActiva(compilacion);
		ModeloEdicion me = edicion.getModeloEdicionActivo( );
		compilacion.compilarFuente( me );
	}

	@Override
	public void compilar ( ) {
		setVistaActiva(compilacion);
		compilacion.compilar( );
	}
	
	@Override
	public void cambiarNombre ( ) {
		Vista v = getVistaActiva( );
		if ( v instanceof InterfazCambiarNombre ) {
			( (InterfazCambiarNombre) v ).cambiarNombre( );
		}
	}
	
	@Override
	public void cambiarDirectorioTrabajo ( ) {
		Vista v = getVistaActiva( );
		if ( v instanceof InterfazCambiarDirectorioTrabajo ) {
			( (InterfazCambiarDirectorioTrabajo) v ).cambiarDirectorioTrabajo( );
		}
	}

	@Override
	public void editarComandoCompilacion ( ) {
		setVistaActiva( compilacion );
		compilacion.editarComando( );
	}

	@Override
	public void añadirFicheros ( ) {
		setVistaActiva( compilacion );
		compilacion.añadirFicheros( );		
	}

	@Override
	public void quitarFicheros ( ) {
		setVistaActiva( compilacion );
		compilacion.quitarFicheros( );		
	}

	@Override
	public void añadirArgumentos ( ) {
		setVistaActiva( ejecucion );
		ejecucion.añadirArgumentos( );		
	}

	@Override
	public void quitarArgumentos ( ) {
		setVistaActiva( ejecucion );
		ejecucion.quitarArgumentos( );
	}

	@Override
	public void añadirEntradas ( ) {
		setVistaActiva( ejecucion );
		ejecucion.añadirEntradas( );
	}

	@Override
	public void añadirFicheroEntradas ( ) {
		setVistaActiva( ejecucion );
		ejecucion.añadirEntradasFichero( );
	}

	@Override
	public void quitarEntradas ( ) {
		setVistaActiva( ejecucion );
		ejecucion.quitarEntradas( );
		
	}

	@Override
	public void ejecutar ( ) {
		setVistaActiva( ejecucion );
		ejecucion.ejecutarPrograma( );
	}

	@Override
	public void ejecutarCompilado ( ) {
		setVistaActiva( ejecucion );
		ModeloCompilacion mo = compilacion.getModeloCompilacionActivo( );
		ejecucion.abrirPrograma( mo );		
	}

	@Override
	public void usarGprofConEjecucion ( ) {
		setVistaActiva( gprof );
		ModeloEjecucion mj = ejecucion.getModeloEjecucionActivo( );
		gprof.abrirDatosEjecucion( mj );
	}

	@Override
	public void gprof ( ) {
		setVistaActiva(gprof);
		gprof.gprof( );
	}

	@Override
	public void editarProgramaGprof ( ) {
		setVistaActiva(gprof);
		gprof.editarProgramaGprof();
	}

	@Override
	public void editarDatosEjecucionGprof ( ) {
		setVistaActiva( gprof );
		gprof.editarDatosEjecucion( );
	}

	@Override
	public void editarCmdG ( ) {
		setVistaActiva( gprof );
		gprof.editarComando();
	}
	
	@Override
	public void pasarAXML ( ) {
		setVistaActiva( traductor );
		ModeloGprof mg = gprof.getModeloGprofActivo( );
		traductor.abrirSalidaEstandarTR( mg );
	}
	
	@Override
	public void editarSalidaGprof ( ) {
		setVistaActiva( traductor );
		traductor.editarSalidaGprof( );
	}
	
	@Override
	public void editarCmdT ( ) {
		setVistaActiva( traductor );
		traductor.editarComando();		
	}
	
	@Override
	public void abrirSalidaEstandarTR ( ) {
		setVistaActiva( traductor );
		traductor.abrirSalidaEstandarTR( );	
	}

	@Override
	public void traducir ( ) {
		setVistaActiva( traductor );
		traductor.traducir( );		
	}

	@Override
	public void verEstadisticas ( ) {
		setVistaActiva( estadisticas );
		ModeloTR mt = traductor.getModeloTRActivo( );
		estadisticas.abrirEstadisticaXML( mt );
	}

	@Override
	public void guardarSesion ( ) {
		Sesion sesion = new Sesion ();
		Vector <ModeloEdicion> Vme = edicion.getModelos();
		sesion.addVMEdicion( Vme );

		Vector <ModeloCompilacion> Vmc = compilacion.getModelos();
		sesion.addVMCompilacion( Vmc );

		Vector <ModeloEjecucion> Vmj = ejecucion.getModelos();
		sesion.addVMEjecucion( Vmj );

		Vector <ModeloGprof> Vmg = gprof.getModelos();
		sesion.addVMGprof( Vmg );

		Vector <ModeloTR> Vmt = traductor.getModelos();
		sesion.addVMTR( Vmt );

		Vector <ModeloEstadistica> Vms = estadisticas.getModelos();
		sesion.addVMEstadistica ( Vms );

		if ( dialogoAbrir.showSaveDialogSesion( this_frame ) ==
			DialogoAbrir.APPROVE_OPTION ) {
			File fichero = dialogoAbrir.getSelectedFile();

			/** Comprobar que el fichero termine con la extensión correcta (".session") */
			if ( !UtilidadesExtensionFichero.isSesion( fichero ) ) {
				fichero = UtilidadesExtensionFichero.addExtension( TipoFichero.SESION, fichero );
			}
			
			/** Nos aseguramos de que el fichero no existe y si existe, 
			 * realizar pregunta de si desea sobreescribir. */
			if ( !fichero.exists( ) || ( fichero.exists() && 
					JOptionPane.showConfirmDialog( guigprof.getInstance( ).getGUI( ) , 
							idiomas.getString( "eti_sesion_seguroSobreescribir" ) , 
							GUI.TITULO_VENTANA, 
							JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION ) ) 
				try {
					guigprof.getInstance( ).guardarSesion( fichero, sesion );

					DialogoError.getInstance( ).showInfoDialog( idiomas.getString( "men_SesionGuardada" ) );

				} catch ( Exception e ) {
					DialogoError.getInstance( ).showErrorDialog( 
							errores.getString( "err_errorEscrituraXMLSesion" ) );
					e.printStackTrace( );
				}
		}
	}

	@Override
	public void acercaDe ( ) {
		new DialogoAcercaDe ( this ).showDialog( );
	}

	@Override
	public void opcionesAplicacion ( ) {
		if ( dialogoPreferencias.showDialog( ) == EstadoDialogoPreferencias.GUARDAR )
			PreferenciasSingleton.getPreferenciasSingleton( ).setPreferencias( dialogoPreferencias.getPreferencias( ) );
	}

	/** función para abrir la sesión. */
	@Override
	public void abrirSesion ( ) {
		File fichero;
		/** Mostrar el dialogo de abrir (DialogoAbrir). */
		
		if ( dialogoAbrir.showOpenDialogSesion( this_frame, false ) == JFileChooser.APPROVE_OPTION ) {
			fichero = dialogoAbrir.getSelectedFile( );

			/** Comprobar fichero seleccionado y abrir una nueva pestaña.  */
			if ( fichero.isFile( ) && fichero.canRead( ) ) {

				int res = DialogoSesion.AÑADIR;
				if( !isVacia() ) {
					DialogoSesion ds = new DialogoSesion( this_frame ); 
					res = ds.showDialog();
				}

				switch( res ) {
				case DialogoSesion.GUARDAR:
					//acciones.getAccionGuardarSesion( ).actionPerformed( null );
					guardarSesion( );
				case DialogoSesion.DESECHAR:
					//acciones.getAccionCerrarTodo( ).actionPerformed( null );
					cerrarTodo( );
				case DialogoSesion.AÑADIR:
					try {
						Sesion sesion = new Sesion ();
						sesion = guigprof.getInstance().leerSesion( fichero );

						Vector <ModeloEdicion> Vme = sesion.getModelosEdicion();
						Iterator < ModeloEdicion > ite = Vme.iterator();
						while (ite.hasNext() ) {
							edicion.abrirEdicion( ite.next() );
						}
						Vector <ModeloCompilacion> Vmc = sesion.getModelosCompilacion();
						Iterator < ModeloCompilacion > itc = Vmc.iterator();
						while (itc.hasNext() ) {
							compilacion.abrirNuevaCompilacion( itc.next() ); //??
						}
						Vector <ModeloEjecucion> Vmj = sesion.getModelosEjecucion();
						Iterator < ModeloEjecucion > itj = Vmj.iterator();
						while (itj.hasNext() ) {
							ejecucion.abrirPrograma( itj.next() );
						}
						Vector <ModeloGprof> Vmg = sesion.getModelosGprof();
						Iterator < ModeloGprof > itg = Vmg.iterator();
						while (itg.hasNext() ) {
							gprof.abrirDatosEjecucion( itg.next() );
						}
						Vector <ModeloTR> Vmt = sesion.getModelosTraduccion();
						Iterator < ModeloTR > itt = Vmt.iterator();
						while (itt.hasNext() ) {
							traductor.abrirSalidaEstandarTR( itt.next() );
						}
						Vector < ModeloEstadistica > Vms = sesion.getModelosEstadisticas();
						Iterator < ModeloEstadistica > its = Vms.iterator();
						while (its.hasNext() ) {
							estadisticas.abrirEstadisticaXML( its.next() );
						}
						
						if ( isNonVista() ) {
							acciones.getAccionVistaEdicion( ).actionPerformed( null );
						}
					}catch ( Exception e ) {
						DialogoError.getInstance( ).showErrorDialog(
								errores.getString( "err_errorLecturaXMLSesion" ) );
					}
					break;

				}

			} else {
				DialogoError.getInstance( ).showErrorDialog( 
						errores.getString( "err_AbrirSesion" ) );
			}
		}
	}

	@Override
	public void editarXML ( ) {
		setVistaActiva(estadisticas);
		estadisticas.editarXML( );
	}

	@Override
	public void verPerfilPlano ( ) {
		setVistaActiva(estadisticas);
		estadisticas.verPerfilPlano( );
	}

	@Override
	public void verGrafoLlamadas ( ) {
		setVistaActiva(estadisticas);
		estadisticas.verGrafoLlamadas( );
	}

	@Override
	public void verFuenteAnotado ( ) {
		setVistaActiva(estadisticas);
		estadisticas.verFuenteAnotado( );
	}

	@Override
	public void verSalidaGprof ( ) {
		setVistaActiva(estadisticas);
		estadisticas.verSalidaGprof( );
	}

}

