package interfaz.dialogos.dialogoPreferencias;

import interfaz.dialogos.dialogoAbrir.DialogoAbrir;
import interfaz.dialogos.dialogoCambiarComando.DialogoCambiarComando;
import interfaz.dialogos.dialogoDirectorio.JDirectoryChooser;
import interfaz.dialogos.dialogoFuente.DialogoFuente;

import java.awt.Dialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import modelo.Preferencias;
import modelo.ficheros.TipoFichero;
import net.miginfocom.swing.MigLayout;

public class DialogoPreferencias extends JDialog {

	/** serial version uid. */
	private static final long serialVersionUID = 8674829277697895228L;

	/** recursos para dar soporte sobre idiomas */
	static ResourceBundle idiomas = ResourceBundle.getBundle("interfaz.idiomas.Recursos");

	/** Preferencias previo */
	Preferencias previo = null;
	/** Preferencias posterior */
	Preferencias posterior = null;
	/** Resultado final de las acciones del usuario. */
	EstadoDialogoPreferencias estado = EstadoDialogoPreferencias.NO_GUARDAR;
	/** Cambios de las preferencias. */
	boolean cambios = false;
	/** Cambios en el look&feel. */
	boolean lyf = false;
	/** Cambios en la fuente. */
	boolean fuente = false;
	
	/** Cadenas sujetas a cambios por idioma. */
	static String				TITULO_DIALOGO			= idiomas
																.getString( "eti_preferencias-dialog" );

	static String				DIRECTORIOS				= idiomas
																.getString( "s_pesDirectorios" );
	static String				ORDENES					= idiomas
																.getString( "s_pesOrdenes" );
	static String				GENERALES				= idiomas
																.getString( "s_pesGenerales" );
	static String				GUARDAR					= idiomas
																.getString( "eti_botGuardar" );
	static String				CANCELAR				= idiomas
																.getString( "eti_botCancelar" );

	static String				MODIFICAR_ORDENES		= idiomas
																.getString( "s_etiModificarOrdenes" );

	static String				COMPILAR				= idiomas
																.getString( "s_ordenCompilar" );
	static String				GPROF					= idiomas
																.getString( "s_ordenGprof" );
	static String				TRADUCTOR				= idiomas
																.getString( "s_ordenTraductor" );
	static String				C						= idiomas
																.getString( "s_ordenC" );
	static String				CPP						= idiomas
																.getString( "s_ordenCpp" );
	static String				PASCAL					= idiomas
																.getString( "s_ordenPascal" );
	static String				FORTRAN					= idiomas
																.getString( "s_ordenFortran" );
	static String				IDIOMA					= idiomas
																.getString( "s_cambiarIdioma" );
	static String				FUENTE					= idiomas
																.getString( "s_cambiarFuente" );
	static String				LOOKaFEEL				= idiomas
																.getString( "s_cambiarLookAFeel" );
	static String				EDITOR					= idiomas
																.getString( "s_cambiarEditor" );
	static String				EDITORxml				= idiomas
																.getString( "s_cambiarEditorXML" );
	static String				MODIFICAR_DIR			= idiomas
																.getString( "s_modificarDir" );

	static String				DIR_COMPILACION			= idiomas
																.getString( "s_dirCompilacion" );
	static String				DIR_EJECUCION			= idiomas
																.getString( "s_dirEjecucion" );
	static String				DIR_GPROF				= idiomas
																.getString( "s_dirGprof" );
	static String				DIR_TRADUCTOR			= idiomas
																.getString( "s_dirTraductor" );
	static String				DIR_GENERAL				= idiomas
																.getString( "s_dirGeneral" );
	static String				ESPAÑOL					= idiomas.getString( "s_español" );
	static String				ENGLISH					= idiomas.getString( "s_english" );


	/** Este panel divide en pestañas las opciones.	 */
	private JTabbedPane pestañas = new JTabbedPane();

	/** Este es el panel de las opciones de comandos */
	private JPanel comandos = new JPanel( new MigLayout( "fill, inset unrel ") );
	
	/** Este es el panel de las opciones de directorios de trabajo. */
	private JPanel dirTrabajo = new JPanel( new MigLayout( "fill, inset unrel ") );
	
	/** Este es el panel de las opciones de generales */
	private JPanel generales = new JPanel( new MigLayout( "fill, inset unrel ") );
	
	/**  Hace falta este this_dialog para usar el this dentro de clases internas. */
	private DialogoPreferencias this_dialog = this;
	
	/** Dialogo para cambiar Fuentes */
	private DialogoFuente diaFuente = new DialogoFuente( this );
	
	/** Dialogo para cambiar ejecutables */
	private DialogoAbrir diaAbrir = new DialogoAbrir( /*this -> no sé si cambiar el comportamiento de 
	dialogo abrir como he hecho con los otros ... */ );
	
	/** Dialogo para cambiar ordenes */
	private DialogoCambiarComando diaOrdenes = new DialogoCambiarComando( this );
	
	/** Dialogo para cambiar directorios de trabajo */
	private JDirectoryChooser diaDirectorio = new JDirectoryChooser ( );

	
	
	/** Botón de cargar las opciones por defecto.	 */
	private JButton botDefault = new JButton( idiomas.getString( "eti_botDefault" ) ) ;
	private ActionListener aclDefault = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			 
			posterior = new Preferencias();
			
			actualizarDialogo();
			
		}
	};
	
	/** Botón de cancelar, cierra el diálogo y anula todas las opciones realizadas hasta ahora.	 */
	private JButton botCancelar = new JButton( idiomas.getString( "eti_botCancelar" ) ) ;
	private ActionListener aclCancelar = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			
			if ( ! ( cambios && JOptionPane.showConfirmDialog( this_dialog , 
					idiomas.getString( "s_seguroCerrarPreferencias" ), TITULO_DIALOGO, JOptionPane.YES_NO_OPTION ) != JOptionPane.YES_OPTION ) ) {
				this_dialog.setVisible( false );
				this_dialog.dispose( );
			}
			
			estado = EstadoDialogoPreferencias.NO_GUARDAR;
		}
	};
	
	/**
	 * Botón de cerrar, cierra el diálogo y confirma todas las opciones realizadas hasta ahora.
	 */
	private JButton botGuardarPreferencias = new JButton( GUARDAR );
	private ActionListener aclGuardar = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			if ( cambios ) {
				estado = EstadoDialogoPreferencias.GUARDAR;
				if ( fuente || lyf )
					JOptionPane.showMessageDialog( this_dialog, idiomas.getString( "s_reinicieParaCambios" ) , 
							TITULO_DIALOGO , JOptionPane.WARNING_MESSAGE );
			}
			this_dialog.setVisible(false);
			this_dialog.dispose( );
		}
	};
	
	/** Añadir los actionListener a los botones. */
	{
		botCancelar.addActionListener( aclCancelar );
		botGuardarPreferencias.addActionListener( aclGuardar );
		botDefault.addActionListener( aclDefault );
	}


	 /** ListBox, con la lista de idiomas disponibles. */
	JComboBox lsbLengua = new JComboBox( new String [] { ESPAÑOL, ENGLISH } );
	private ActionListener aclLengua = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			String strLengua = lsbLengua.getSelectedItem( ).toString( );
			Locale l = null;
			if( strLengua.equals( ESPAÑOL ) )
				l = new Locale( "es", "ES" );
			else if ( strLengua.equals( ENGLISH ) ) 
				l = new Locale( "en", "UK" );
			
			if ( !previo.getLocale( ).equals( l ) ) {
				posterior.setLocale( l );
				cambios = true;
				fuente = true;
			}
			/*				if ( JOptionPane.showConfirmDialog( this_dialog , idiomas.getString( "s_seguroInstalarL&F" ),
			TITULO_DIALOGO , JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION )
		estado = EstadoDialogoPreferencias.GUARDAR_Y_APLICAR;*/
		}
	};

	{
		lsbLengua.addActionListener( aclLengua );
	}
	

	/** ListBox, con la lista de Look & Feel disponibles. */
	LookAndFeelInfo [] lafinfo = UIManager.getInstalledLookAndFeels( );
	
	JComboBox lsbLookAfeel;
	{
		Vector <String> lyf = new Vector <String> ( lafinfo.length );
		
		/* este es el elemento que representa que no hay especificado ninguno concreto. */
		lyf.addElement( "" );
		
		/* se añaden el resto de elementos de laf instalados */
		for ( int i = 0 ; i < lafinfo.length ; i ++ ) 
			lyf.addElement( lafinfo[i].getName( ) );
		
		lsbLookAfeel = new JComboBox ( lyf );
	}
	
	private ActionListener aclLookAfeel = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			if ( this_dialog.isVisible( ) ) {
				Object sel = lsbLookAfeel.getSelectedItem(  );
				if ( sel != null ) {
					String lafElegido = sel.toString();

					if( lafElegido != null ) {
						if ( !posterior.getLookAndFeelName( ).equals( lafElegido ) ) {
							posterior.setLookAndFeelName( lafElegido );
							if ( !posterior.getLookAndFeelName( ).equals( previo.getLookAndFeelName( ) ) ) {
								cambios = true;
								lyf = true;
							}
							
						}

						/*				if ( JOptionPane.showConfirmDialog( this_dialog , idiomas.getString( "s_seguroInstalarL&F" ),
						TITULO_DIALOGO , JOptionPane.YES_NO_OPTION ) == JOptionPane.YES_OPTION )
					estado = EstadoDialogoPreferencias.GUARDAR_Y_APLICAR;*/
					}
				}
			}
		}
	};
	{
		lsbLookAfeel.addActionListener( aclLookAfeel );
	}

	/** Opción para la fuente usada */
	private JTextField	txtFuente = new JTextField();
	{
		txtFuente.setEditable( false );
	}
	
	private JButton	botCambiarFuente = new JButton ( "..." );
	private ActionListener aclCambiarFuente = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			Font fuenteActual = posterior.getEstadisticasFont( );
			
			diaFuente.setFuente( fuenteActual );
			
			final boolean d = diaFuente.showDialog( );
			
			Font fuenteDespuesDialogo = diaFuente.getFuente( );
			if ( d && !fuenteDespuesDialogo.equals( fuenteActual ) ) {
				posterior.setEstadisticasFont( fuenteDespuesDialogo );
				
				/*txtFuente.setText( fuenteDespuesDialogo.getFontName( ) + " " + 
						String.valueOf( fuenteDespuesDialogo.getSize( ) ) );*/
				/*txtFuente.setText( posterior.getEstadisticasFont( ).toString( ) );*/
				txtFuente.setText( stringFuente( posterior.getEstadisticasFont( ) ) );

				if ( !previo.getEstadisticasFont( ).equals( posterior.getEstadisticasFont( ) ) ) {
					cambios = true;
				}
			}
		}
	};
	{
		botCambiarFuente.addActionListener( aclCambiarFuente );
	}
	

	/** Opción para el editor de ficheros fuente. */
	private JTextField	txtEditor = new JTextField(  );
	private DocumentListener		dolEditor			= new DocumentListener( ) {

			public void changes (
					DocumentEvent docEv ) {
				if ( !posterior.getEditor().equals(txtEditor.getText( ) ) ) {
					posterior.setEditor( txtEditor.getText( ) );
					if ( !previo.getEditor( ).equals( posterior.getEditor( ) ) )
						this_dialog.cambios = true;
				}
			}

			@Override
			public void changedUpdate (
					DocumentEvent docEv ) {
				changes(docEv);
			}

			@Override
			public void insertUpdate (
					DocumentEvent docEv ) {
				changes( docEv );

			}

			@Override
			public void removeUpdate (
					DocumentEvent docEv ) {
				changes( docEv );

			}
		};
	{
		/** Añadir el listener del JTextField. */
		txtEditor.getDocument( ).addDocumentListener( dolEditor );
	}

	private JButton	botCambiarEditor = new JButton ( "..." );
	private ActionListener aclCambiarEditor = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			diaAbrir.showOpenDialogEjecucion( this_dialog , false );
			
			File editor = diaAbrir.getSelectedFile( );
			if ( editor != null ) {				
				String s_editor = new String();
				try {
					s_editor = editor.getCanonicalPath( );
				} catch ( IOException e1 ) {
					s_editor = editor.getAbsolutePath();
				}
				
				if ( !s_editor.equals( txtEditor.getText( ) ) ) {
					txtEditor.setText( s_editor );
					cambios = true;
					posterior.setEditor( s_editor );
					
					if ( !s_editor.equals( previo.getEditor( ) ) ) 
						cambios = true;
				}
			}
		}
	};
	{
		botCambiarEditor.addActionListener( aclCambiarEditor );
	}

	/** Opción para el editor XML. */
	private JTextField	txtEditorXML = new JTextField( );
	private DocumentListener		dolEditorXML			= new DocumentListener( ) {

		public void changes (
				DocumentEvent docEv ) {
			if ( !posterior.getEditorXML().equals(txtEditorXML.getText( ) ) ) {
				posterior.setEditorXML( txtEditorXML.getText( ) );
				if ( !previo.getEditorXML( ).equals( posterior.getEditorXML( ) ) )
					this_dialog.cambios = true;
			}
		}

		@Override
		public void changedUpdate (
				DocumentEvent docEv ) {
			changes(docEv);
		}

		@Override
		public void insertUpdate (
				DocumentEvent docEv ) {
			changes( docEv );

		}

		@Override
		public void removeUpdate (
				DocumentEvent docEv ) {
			changes( docEv );

		}
	};
	{
		/** Añadir el listener del JTextField. */
		txtEditorXML.getDocument( ).addDocumentListener( dolEditorXML );
	}
	
	private JButton	botCambiarEditorXML = new JButton ( "..." );
	private ActionListener aclCambiarEditorXML = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
diaAbrir.showOpenDialogEjecucion( this_dialog , false );
			
			File editorXML = diaAbrir.getSelectedFile( );
			if ( editorXML != null ) {				
				String s_editorXML = new String();
				try {
					s_editorXML = editorXML.getCanonicalPath( );
				} catch ( IOException e1 ) {
					s_editorXML = editorXML.getAbsolutePath();
				}
				
				if ( !s_editorXML.equals( txtEditorXML.getText( ) ) ) {
					txtEditorXML.setText( s_editorXML );
					posterior.setEditorXML( s_editorXML );
					
					if ( !s_editorXML.equals( previo.getEditorXML( ) ) ) 
						cambios = true;
				}
			}
		}
	};
	{
		botCambiarEditorXML.addActionListener( aclCambiarEditorXML );
	}

	private JTextField	txtDirGeneral = new JTextField( );
	{
		txtDirGeneral.setEditable( false );
	}

	private JButton	botCambiarDirGeneral = new JButton ( "..." );
	private ActionListener aclCambiarDirGeneral = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			diaDirectorio.showDialog( this_dialog );
			
			File dir = diaDirectorio.getSelectedDirectory( );
			if ( dir != null && dir.isDirectory( )  ) { 
				
				String s_dir = new String();
				try {
					s_dir = dir.getCanonicalPath( );
				} catch ( IOException e1 ) {
					s_dir = dir.getAbsolutePath();
				}
					
				if ( !s_dir.equals( txtDirGeneral.getText( ) ) ) {
					txtDirGeneral.setText( s_dir );
					posterior.setWorkspaceDefecto( dir );
					
					if ( !dir.equals( previo.getWorkspaceDefecto( ) ) ) 
						cambios = true;
				}
			}
		}
	};
	{
		botCambiarDirGeneral.addActionListener( aclCambiarDirGeneral );
	}

	private JTextField	txtDirCompilacion = new JTextField( );
	{
		txtDirCompilacion.setEditable( false );
	}
	private JButton	botCambiarDirCompilacion = new JButton ( "..." );
	private ActionListener aclCambiarDirCompilacion = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			diaDirectorio.showDialog( this_dialog );
			
			File dir = diaDirectorio.getSelectedDirectory( );
			if ( dir != null && dir.isDirectory( )  ) { 
				
				String s_dir = new String();
				try {
					s_dir = dir.getCanonicalPath( );
				} catch ( IOException e1 ) {
					s_dir = dir.getAbsolutePath();
				}
				
				if ( !s_dir.equals( txtDirCompilacion.getText( ) ) ) {
					txtDirCompilacion.setText( s_dir );
					posterior.setWorkspaceDefecto( dir );
					
					if ( !dir.equals( previo.getWorkspaceDefecto( ) ) ) 
						cambios = true;
				}
			}
		}
	};
	{
		botCambiarDirCompilacion.addActionListener( aclCambiarDirCompilacion );
	}
	
	private JTextField	txtDirEjecucion = new JTextField( );
	{
		txtDirEjecucion.setEditable( false );
	}
	
	private JButton	botCambiarDirEjecucion = new JButton ( "..." );
	private ActionListener aclCambiarDirEjecucion = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			diaDirectorio.showDialog( this_dialog );
			
			File dir = diaDirectorio.getSelectedDirectory( );
			if ( dir != null && dir.isDirectory( )  ) { 
				
				String s_dir = new String();
				try {
					s_dir = dir.getCanonicalPath( );
				} catch ( IOException e1 ) {
					s_dir = dir.getAbsolutePath();
				}
					
				if ( !s_dir.equals( txtDirEjecucion.getText( ) ) ) {
					txtDirEjecucion.setText( s_dir );
					posterior.setWorkspaceDefecto( dir );
					
					if ( !dir.equals( previo.getWorkspaceDefecto( ) ) ) 
						cambios = true;
				}
			}
		}
	};
	{
		botCambiarDirEjecucion.addActionListener( aclCambiarDirEjecucion );
	}
	

	private JTextField	txtDirGprof = new JTextField( );
	{
		txtDirGprof.setEditable( false );
	}
	
	private JButton	botCambiarDirGprof = new JButton ( "..." );
	private ActionListener aclCambiarDirGprof = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			diaDirectorio.showDialog( this_dialog );
			
			File dir = diaDirectorio.getSelectedDirectory( );
			if ( dir != null && dir.isDirectory( )  ) { 
				
				String s_dir = new String();
				try {
					s_dir = dir.getCanonicalPath( );
				} catch ( IOException e1 ) {
					s_dir = dir.getAbsolutePath();
				}
				
				if ( !s_dir.equals( txtDirGprof.getText( ) ) ) {
					txtDirGprof.setText( s_dir );
					posterior.setWorkspaceDefecto( dir );
					
					if ( !dir.equals( previo.getWorkspaceDefecto( ) ) ) 
						cambios = true;
				}
			}
		}
	};
	{
		botCambiarDirGprof.addActionListener( aclCambiarDirGprof );
	}

	private JTextField	txtDirTraductor = new JTextField( );
	{
		txtDirTraductor.setEditable( false );
	}
	
	private JButton	botCambiarDirTraductor = new JButton ( "..." );
	private ActionListener aclCambiarDirTraductor = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			diaDirectorio.showDialog( this_dialog );
			
			File dir = diaDirectorio.getSelectedDirectory( );
			if ( dir != null && dir.isDirectory( )  ) { 
				
				String s_dir = new String();
				try {
					s_dir = dir.getCanonicalPath( );
				} catch ( IOException e1 ) {
					s_dir = dir.getAbsolutePath();
				}
				
				if ( !s_dir.equals( txtDirTraductor.getText( ) ) ) {
					txtDirTraductor.setText( s_dir );
					posterior.setWorkspaceDefecto( dir );
					
					if ( !dir.equals( previo.getWorkspaceDefecto( ) ) ) 
						cambios = true;
				}
			}
		}
	};
	{
		botCambiarDirTraductor.addActionListener( aclCambiarDirTraductor );
	}

	private JTextField	txtOrdenCompilar = new JTextField( );
	{
		txtOrdenCompilar.setEditable( false );
	}
	
	private JButton	botCambiarOrdenCompilar = new JButton ( "..." );
	private ActionListener aclCambiarOrdenCompilar = new ActionListener () {
		public void actionPerformed( ActionEvent e) {

			
			diaOrdenes.setComando( posterior.getComandoCompilar( ) );

			boolean ordenModificada = diaOrdenes.showDialog( );
			
			Vector <String> orden = diaOrdenes.getComando();
			
			if ( ordenModificada && orden != null && orden.size( ) > 0 ) {
				
				String ordenPlana = orden.toString( );
				
				if ( !ordenPlana.equals( txtOrdenCompilar.getText( ) ) ) {
					txtOrdenCompilar.setText( ordenPlana );
					if ( orden.size( ) > 0 )
						posterior.setCompilador( orden.firstElement( ) );
					if ( orden.size( ) > 1 )
						posterior.setOpcionesCompilador( orden.subList( 1 , orden.size( ) - 1  ) );
					
					if ( !orden.equals( previo.getComandoCompilar( ) ) ) 
						cambios = true;
				}
				
			}
		}
	};
	{
		botCambiarOrdenCompilar.addActionListener( aclCambiarOrdenCompilar );
	}

	private JTextField	txtOrdenC = new JTextField( );
	{
		txtOrdenC.setEditable( false );
	}
	
	private JButton	botCambiarOrdenC = new JButton ( "..." );
	private ActionListener aclCambiarOrdenC = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
				
			diaOrdenes.setComando( posterior.getComandoCompilar( TipoFichero.C ) );
			
			boolean ordenModificada = diaOrdenes.showDialog( );
			
			Vector <String> orden = diaOrdenes.getComando();
			
			if ( ordenModificada && orden != null && orden.size( ) > 0 ) {
				
				String ordenPlana = orden.toString( );
										
				if ( !ordenPlana.equals( txtOrdenC.getText( ) ) ) {
					txtOrdenC.setText( ordenPlana );
					if ( orden.size( ) > 0 )
						posterior.setCompilador( orden.firstElement( ), TipoFichero.C );
					if ( orden.size( ) > 1 )
						posterior.setOpcionesCompilador( orden.subList( 1 , orden.size( ) - 1  ), TipoFichero.C );
					
					if ( !orden.equals( previo.getComandoCompilar( TipoFichero.C ) ) ) 
						cambios = true;
				}
			}
		}
	};
	{
		botCambiarOrdenC.addActionListener( aclCambiarOrdenC );
	}

	private JTextField	txtOrdenCpp = new JTextField( );
	{
		txtOrdenCpp.setEditable( false );
	}
	
	private JButton	botCambiarOrdenCpp = new JButton ( "..." );
	private ActionListener aclCambiarOrdenCpp = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
				
			diaOrdenes.setComando( posterior.getComandoCompilar( TipoFichero.CPP ) );
			
			boolean ordenModificada = diaOrdenes.showDialog( );
			
			Vector <String> orden = diaOrdenes.getComando();
			
			if ( ordenModificada && orden != null && orden.size( ) > 0 ) {
				
				String ordenPlana = orden.toString( );
				
				if ( !ordenPlana.equals( txtOrdenCpp.getText( ) ) ) {
					txtOrdenCpp.setText( ordenPlana );
					if ( orden.size( ) > 0 )
						posterior.setCompilador( orden.firstElement( ), TipoFichero.CPP );
					if ( orden.size( ) > 1 )
						posterior.setOpcionesCompilador( orden.subList( 1 , orden.size( ) - 1  ), TipoFichero.CPP );
					
					if ( !orden.equals( previo.getComandoCompilar( TipoFichero.CPP ) ) ) 
						cambios = true;
				}
			}
		}
	};
	{
		botCambiarOrdenCpp.addActionListener( aclCambiarOrdenCpp );
	}

	private JTextField	txtOrdenPascal = new JTextField( );
	{
		txtOrdenPascal.setEditable( false );
	}
	
	private JButton	botCambiarOrdenPascal = new JButton ( "..." );
	private ActionListener aclCambiarOrdenPascal = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			
			diaOrdenes.setComando( posterior.getComandoCompilar( TipoFichero.PASCAL ) );
			
			boolean ordenModificada = diaOrdenes.showDialog( );
			
			Vector <String> orden = diaOrdenes.getComando();
			
			if ( ordenModificada && orden != null && orden.size( ) > 0 ) {
				
				String ordenPlana = orden.toString( );
				
				if ( !ordenPlana.equals( txtOrdenPascal.getText( ) ) ) {
					txtOrdenPascal.setText( ordenPlana );
					if ( orden.size( ) > 0 )
						posterior.setCompilador( orden.firstElement( ), TipoFichero.PASCAL );
					if ( orden.size( ) > 1 )
						posterior.setOpcionesCompilador( orden.subList( 1 , orden.size( ) - 1  ), TipoFichero.PASCAL );
					
					if ( !orden.equals( previo.getComandoCompilar( TipoFichero.PASCAL ) ) ) 
						cambios = true;
				}	
			}
		}
	};
	{
		botCambiarOrdenPascal.addActionListener( aclCambiarOrdenPascal );
	}

	private JTextField	txtOrdenFortran = new JTextField( );
	{
		txtOrdenFortran.setEditable( false );
	}
	
	private JButton	botCambiarOrdenFortran = new JButton ( "..." );
	private ActionListener aclCambiarOrdenFortran = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
				
			diaOrdenes.setComando( posterior.getComandoCompilar( TipoFichero.FORTRAN ) );
			
			boolean ordenModificada = diaOrdenes.showDialog( );
			
			Vector <String> orden = diaOrdenes.getComando();
			
			if ( ordenModificada && orden != null && orden.size( ) > 0 ) {
				
				String ordenPlana = orden.toString( );
				
				if ( !ordenPlana.equals( txtOrdenFortran.getText( ) ) ) {
					txtOrdenFortran.setText( ordenPlana );
					if ( orden.size( ) > 0 )
						posterior.setCompilador( orden.firstElement( ), TipoFichero.FORTRAN );
					if ( orden.size( ) > 1 )
						posterior.setOpcionesCompilador( orden.subList( 1 , orden.size( ) - 1  ), TipoFichero.FORTRAN );
					
					if ( !orden.equals( previo.getComandoCompilar( TipoFichero.FORTRAN ) ) ) 
						cambios = true;
				}	
			}
		}
	};
	{
		botCambiarOrdenFortran.addActionListener( aclCambiarOrdenFortran );
	}

	private JTextField	txtOrdenGprof = new JTextField(  );
	{
		txtOrdenGprof.setEditable( false );
	}
	
	private JButton	botCambiarOrdenGprof = new JButton ( "..." );
	private ActionListener aclCambiarOrdenGprof = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			
			diaOrdenes.setComando( posterior.getComandoGprof( ) );
				
			boolean ordenModificada = diaOrdenes.showDialog( );
			
			Vector <String> orden = diaOrdenes.getComando();
			
			if ( ordenModificada && orden != null && orden.size( ) > 0 ) {
				
				String ordenPlana = orden.toString( );
				
				if ( !ordenPlana.equals( txtOrdenGprof.getText( ) ) ) {
					txtOrdenGprof.setText( ordenPlana );
					if ( orden.size( ) > 0 )
						posterior.setCompilador( orden.firstElement( ), TipoFichero.GPROF );
					if ( orden.size( ) > 1 )
						posterior.setOpcionesCompilador( orden.subList( 1 , orden.size( ) - 1  ), TipoFichero.GPROF );
					
					if ( !orden.equals( previo.getComandoCompilar( TipoFichero.GPROF ) ) ) 
						cambios = true;
				}						
			}
		}
	};
	{
		botCambiarOrdenGprof.addActionListener( aclCambiarOrdenGprof );
	}

	private JTextField	txtOrdenTraductor = new JTextField( );
	{
		txtOrdenTraductor.setEditable( false );
	}
	
	private JButton	botCambiarOrdenTraductor = new JButton ( "..." );
	private ActionListener aclCambiarOrdenTraductor = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
				
			diaOrdenes.setComando( posterior.getComandoTraductor( ) );
			
			boolean ordenModificada = diaOrdenes.showDialog( );
			
			Vector <String> orden = diaOrdenes.getComando();
			
			if ( ordenModificada && orden != null && orden.size( ) > 0 ) {
				
				String ordenPlana = orden.toString( );
				
				if ( !ordenPlana.equals( txtOrdenTraductor.getText( ) ) ) {
					txtOrdenTraductor.setText( ordenPlana );
					if ( orden.size( ) > 0 )
						posterior.setCompilador( orden.firstElement( ), TipoFichero.GPROFXML );
					if ( orden.size( ) > 1 )
						posterior.setOpcionesCompilador( orden.subList( 1 , orden.size( ) - 1  ), TipoFichero.GPROFXML );
					
					if ( !orden.equals( previo.getComandoCompilar( TipoFichero.GPROFXML ) ) ) 
						cambios = true;
				}						
			}
		}
	};
	{
		botCambiarOrdenTraductor.addActionListener( aclCambiarOrdenTraductor );
	}
	
	
	/** Constructores de la clase 
	 * @param owner
	 */
	public DialogoPreferencias(Frame owner, Preferencias preferencias ) {
		/** ventana padre, título del diálogo y que sea modal */
		super( owner , TITULO_DIALOGO , true );

		setPreferencias ( preferencias );
		
		init();	
	}
	public DialogoPreferencias(Dialog owner, Preferencias preferencias ) {
		/** ventana padre, título del diálogo y que sea modal */
		super( owner , TITULO_DIALOGO , true );

		setPreferencias ( preferencias );
		
		init();	
	}
	public DialogoPreferencias(Window owner, Preferencias preferencias ) {
		/** ventana padre, título del diálogo y que sea modal */
		super( owner , TITULO_DIALOGO , Dialog.ModalityType.APPLICATION_MODAL );

		setPreferencias ( preferencias );
		
		init();	
	}
	
	public DialogoPreferencias( Dialog owner ) {
		/** ventana padre, título del diálogo y que sea modal */
		super( owner , TITULO_DIALOGO , true );

		setPreferencias ( new Preferencias() );
		
		init();	
	}
	public DialogoPreferencias( Window owner ) {
		/** ventana padre, título del diálogo y que sea modal */
		super( owner , TITULO_DIALOGO , Dialog.ModalityType.APPLICATION_MODAL );

		setPreferencias ( new Preferencias() );
		
		init();	
	}


	private void init () {

		//this.setDefaultCloseOperation( DISPOSE_ON_CLOSE );
		this.setDefaultCloseOperation( DO_NOTHING_ON_CLOSE );

		/** colocar elementos del diálogo. */
		this.setLayout( new MigLayout( "fill, inset unrel" ) );
		
		this.add( pestañas , "wrap" );
		
		pestañas.addTab( GENERALES, this.generales );
		pestañas.addTab( DIRECTORIOS, this.dirTrabajo );
		pestañas.addTab( ORDENES, this.comandos );
		
		generales.add( new JLabel( IDIOMA ), "gapafter rel, align right, sgy alto" );
		generales.add( lsbLengua, "sgy alto, growx, pushx, span, wrap" );
		generales.add( new JLabel( LOOKaFEEL ), "gapafter rel, align right, sgy alto" );
		generales.add( lsbLookAfeel, "sgy alto, growx, pushx, span, wrap" );
		generales.add( new JLabel( FUENTE ), "gapafter rel, align right, sgy alto" );
		generales.add( txtFuente, "sgy alto, growx, pushx, wmin 160pt" );
		generales.add( botCambiarFuente, "sgy alto, wrap" );
		generales.add( new JLabel( EDITOR ), "gapafter rel, align right, sgy alto" );
		generales.add( txtEditor, "sgy alto, growx, pushx" );
		generales.add( botCambiarEditor, "sgy alto, wrap" );
		generales.add( new JLabel( EDITORxml ), "gapafter rel, align right, sgy alto" );
		generales.add( txtEditorXML, "sgy alto, growx, pushx" );
		generales.add( botCambiarEditorXML, "sgy alto, wrap" );
		
		dirTrabajo.add( new JLabel( DIR_GENERAL ), "gapafter rel, align right, sgy alto" );
		dirTrabajo.add( txtDirGeneral, "sgy alto, growx, pushx" );
		dirTrabajo.add( botCambiarDirGeneral, "sgy alto, wrap" );
		dirTrabajo.add( new JLabel( DIR_COMPILACION ), "gapafter rel, align right, sgy alto" );
		dirTrabajo.add( txtDirCompilacion, "sgy alto, growx, pushx" );
		dirTrabajo.add( botCambiarDirCompilacion, "sgy alto, wrap" );
		dirTrabajo.add( new JLabel( DIR_EJECUCION ), "gapafter rel, align right, sgy alto" );
		dirTrabajo.add( txtDirEjecucion, "sgy alto, growx, pushx" );
		dirTrabajo.add( botCambiarDirEjecucion, "sgy alto, wrap" );
		dirTrabajo.add( new JLabel( DIR_GPROF ), "gapafter rel, align right, sgy alto" );
		dirTrabajo.add( txtDirGprof, "sgy alto, growx, pushx" );
		dirTrabajo.add( botCambiarDirGprof, "sgy alto, wrap" );
		dirTrabajo.add( new JLabel( DIR_TRADUCTOR ), "gapafter rel, align right, sgy alto" );
		dirTrabajo.add( txtDirTraductor, "sgy alto, growx, pushx" );
		dirTrabajo.add( botCambiarDirTraductor, "sgy alto, wrap" );
		
		comandos.add( new JLabel( COMPILAR ), "gapafter rel, align right, sgy alto" );
		comandos.add( txtOrdenCompilar, "sgy alto, growx, pushx" );
		comandos.add( botCambiarOrdenCompilar, "sgy alto, wrap" );
		comandos.add( new JLabel( C ), "gapafter rel, align right, sgy alto" );
		comandos.add( txtOrdenC, "sgy alto, growx, pushx" );
		comandos.add( botCambiarOrdenC, "sgy alto, wrap" );
		comandos.add( new JLabel( CPP ), "gapafter rel, align right, sgy alto" );
		comandos.add( txtOrdenCpp, "sgy alto, growx, pushx" );
		comandos.add( botCambiarOrdenCpp, "sgy alto, wrap" );
		comandos.add( new JLabel( PASCAL ), "gapafter rel, align right, sgy alto" );
		comandos.add( txtOrdenPascal, "sgy alto, growx, pushx" );
		comandos.add( botCambiarOrdenPascal, "sgy alto, wrap" );
		comandos.add( new JLabel( FORTRAN ), "gapafter rel, align right, sgy alto" );
		comandos.add( txtOrdenFortran, "sgy alto, growx, pushx" );
		comandos.add( botCambiarOrdenFortran, "sgy alto, wrap" );
		comandos.add( new JLabel( GPROF ), "gapafter rel, align right, sgy alto" );
		comandos.add( txtOrdenGprof, "sgy alto, growx, pushx" );
		comandos.add( botCambiarOrdenGprof, "sgy alto, wrap" );
		comandos.add( new JLabel( TRADUCTOR ), "gapafter rel, align right, sgy alto" );
		comandos.add( txtOrdenTraductor, "sgy alto, growx, pushx" );
		comandos.add( botCambiarOrdenTraductor, "sgy alto, wrap" );
		

		this.add( this.botDefault , 
				"gaptop unrel, span, alignx center, sg botones" );
		this.add( botGuardarPreferencias ,
				"gaptop rel, span, split 2, alignx center, sg botones" );
		this.add( this.botCancelar , "sg botones" );

		/** default button -> Guardar */
		this.getRootPane( ).setDefaultButton( botGuardarPreferencias );

		/** ESC button -> Cancelar */
		KeyStroke esc = KeyStroke.getKeyStroke( KeyEvent.VK_ESCAPE , 0 );
		this.getRootPane( ).registerKeyboardAction( aclCancelar ,
				"ACTION_CLOSE" , esc , JComponent.WHEN_IN_FOCUSED_WINDOW );
		
		/** cerrar Ventana -> Cancelar */ 
		this.addWindowListener(new WindowAdapter() {
	        @Override public void windowClosing(WindowEvent e) {
	            botCancelar.doClick( );
	        }
	    });


		/** Empaquetar y colocar */
		this.pack( );
		this.setLocationRelativeTo( this.getParent() );
		this.setResizable( false );	

	}
		
	public EstadoDialogoPreferencias showDialog ( ) {
		estado = EstadoDialogoPreferencias.NO_GUARDAR;
		this.setVisible( true );
		this.setLocationRelativeTo( this.getParent( ) );
		this.setResizable( false );
		
		return estado;
	}
	
	private void actualizarDialogo() {
		/** lista desplegable de look and feel */
		{
			/*LookAndFeel actualLYF = UIManager.getLookAndFeel( ); 
			String seleccionado =  actualLYF == null? "": actualLYF.getName( );*/
			String lyf_preferencias = posterior.getLookAndFeelName( );
			
			/* En principio se señala el indefinido "", que está en el 0. */
			int sel = 0;
			
			if ( !lyf_preferencias.isEmpty( ) ) {
				for ( int i = 0 ; i < lafinfo.length ; i++ ) {
					String lyf_instalado = lafinfo[i].getName( );
					if ( sel < 0 && lyf_instalado.equals( lyf_preferencias ) ) {
						sel = i + 1;
						/* El +1 es porque los lyf están a partir del 1. La
							posición 0 está reservada para el indefinido "". */
					}
				}

				/* Si no se encuentra el definido en las preferencias entre los
				  instalados, éste, se debe añadir en la lista. */
				if ( sel == 0 ) {
					lsbLookAfeel.addItem( lyf_preferencias );
					sel = lsbLookAfeel.getItemCount( ) - 1;
				}
			}
			if( sel >= 0 && sel < lsbLookAfeel.getItemCount( ) ) {
				lsbLookAfeel.setSelectedIndex( sel );
			}
		}
		
		/** Locale */
		{
			if ( posterior.getLocale( ).getCountry( ).equals( "uk" ) )
				this.lsbLengua.setSelectedItem( ENGLISH );
			else
				this.lsbLengua.setSelectedItem( ESPAÑOL );
		}
		
		/** Fuente */
		{
			Font fontFuente = posterior.getEstadisticasFont( );
			txtFuente.setText( stringFuente(fontFuente) ); 
			/*txtFuente.setText( 	fontFuente.getFontName( ) + " " + 
				String.valueOf( fontFuente.getSize( ) ) );*/
			/*txtFuente.setText( posterior.getEstadisticasFont( ).toString( ) );*/
		}

	
		/** Directorios */
		txtDirCompilacion.setText (
				posterior.getWorkspaceCompilacion( ).getAbsolutePath( ).toString( ) );
		txtDirEjecucion.setText (
				posterior.getWorkspaceEjecucion( ).getAbsolutePath( ).toString( ) );
		txtDirGprof.setText (
				posterior.getWorkspaceGprof( ).getAbsolutePath( ).toString( ) );
		txtDirTraductor.setText (
				posterior.getWorkspaceTR( ).getAbsolutePath( ).toString( ) );
		txtDirGeneral.setText (
				posterior.getWorkspaceDefecto( ).getAbsolutePath( ).toString( ) );
		
		/** Ordenes */
		txtEditor.setText(
				posterior.getEditor( ) );
	
		txtEditorXML.setText( 
				posterior.getEditorXML( ) );

		
		txtOrdenCompilar.setText (
				posterior.getComandoCompilar( ).toString( ) );
		txtOrdenTraductor.setText (
				posterior.getComandoTraductor( ).toString() );
		txtOrdenC.setText (
				posterior.getComandoC( ).toString( ) );
		txtOrdenCpp.setText (
				posterior.getComandoCpp( ).toString( ) );
		txtOrdenPascal.setText (
				posterior.getComandoPascal( ).toString( ) );
		txtOrdenFortran.setText (
				posterior.getComandoFortran( ).toString() );
		txtOrdenGprof.setText (
				posterior.getComandoGprof( ).toString() );
		
	}
	
	/** devuelve las preferencias modificadas. */ 
	public Preferencias getPreferencias() {
		return posterior;
	}
	
	/** 
	
	/** setear las preferencias */
	public void setPreferencias ( Preferencias preferencias ) {
		previo = preferencias;
		
		posterior = (Preferencias) previo.clone();
		
		cambios = false;
		
		actualizarDialogo();
	}
	
	/** Función main para pruebas. */
	public static void main ( String [] args ) {
		javax.swing.JFrame v = new javax.swing.JFrame( TITULO_DIALOGO );
		javax.swing.JPanel p = new javax.swing.JPanel( );
		javax.swing.JButton b = new javax.swing.JButton(
				"Mostrar diálogo Preferencias" );
		p.add( b );
		v.setContentPane( p );
		v.pack( );
		final DialogoPreferencias d = new DialogoPreferencias( v , new Preferencias() );
		;

		b.addActionListener( new ActionListener( ) {
			public void actionPerformed ( ActionEvent e ) {
				d.showDialog( );
			}
		} );

		v.setDefaultCloseOperation( javax.swing.JFrame.EXIT_ON_CLOSE );
		v.setVisible( true );
	}

	private String stringFuente( Font f ) {
		String s = new String();
			
		String estilo = new String();
		switch ( f.getStyle( ) ) {
		case 0 :
			estilo = idiomas.getString( "s_normal" );;
			break;
		case 1 :
			estilo = idiomas.getString( "s_negrita" );;
			break;
		case 2 :
			estilo = idiomas.getString( "s_italica" );;
			break;
		case 3 :
			estilo = idiomas.getString( "s_italicaYnegrita" );;
			break;
		}
		
		
		s = f.getName( ) + " " +
		f.getSize( ) + " " +
		 estilo;
		
		return s;
	}
}
