package interfaz.dialogos.error;

import interfaz.dialogos.JMessageText;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ResourceBundle;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.KeyStroke;
import javax.swing.UIManager;

import negocio.PreferenciasSingleton;
import net.miginfocom.swing.MigLayout;


public class DialogoError extends JDialog {
	/** serial version uid. */
	private static final long serialVersionUID = 1L;
	
	/** recursos para dar soporte sobre idiomas */
	static ResourceBundle idiomas = ResourceBundle.getBundle("interfaz.idiomas.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	
	/** Esta propiedad sirve para implementar el singleton. */
    private static DialogoError INSTANCE = null;

	
	static String s_Aceptar =  idiomas.getString( "eti_botOk" );
	static String s_tituloDialogo = idiomas.getString( "eti_error-dialog");
	
	/** Botón ACEPTAR */
	private JButton botAceptar = new JButton( s_Aceptar );
	private ActionListener aclAceptar = new ActionListener () {
		public void actionPerformed( ActionEvent e) {
			INSTANCE.setVisible( false );
		}
	};
	{ /** Añadir los actionListener a los botones. */
		botAceptar.addActionListener( aclAceptar );
	}
	
	
	/** Pregunta antes de cerrar */
	private JMessageText mensaje = new JMessageText ( "" );
	
	/** Icono */
	static Icon errorIcon = UIManager.getIcon( "OptionPane.errorIcon" );
	static Icon infoIcon = UIManager.getIcon( "OptionPane.informationIcon" );
	private JLabel icono = new JLabel( errorIcon );
	
	public void showErrorDialog( String m ) {
		mensaje.setText( m );
		icono.setIcon( errorIcon );
		this.pack( );
		this.setModalityType( JDialog.ModalityType.APPLICATION_MODAL  );
		this.setLocationRelativeTo( this.getParent( ) );
		this.setDefaultCloseOperation( DISPOSE_ON_CLOSE );
		this.setVisible( true );
	}
	
	public void showInfoDialog( String m ) {
		mensaje.setText( m );
		icono.setIcon( infoIcon );
		this.pack( );
		this.setModalityType( JDialog.ModalityType.APPLICATION_MODAL  );
		this.setLocationRelativeTo( this.getParent( ) );
		this.setDefaultCloseOperation( DISPOSE_ON_CLOSE );
		this.setVisible( true );
	}
	
    // creador sincronizado para protegerse de posibles problemas  multi-hilo
    // otra prueba para evitar instanciación múltiple 
    private synchronized static void createInstance( Frame owner ) {
        if (INSTANCE == null) { 
            INSTANCE = new DialogoError ( owner );
        } else {
        	if ( INSTANCE.getOwner( ) != owner ) {
        		INSTANCE = new DialogoError ( owner );
        	}
        }
    }

    public static DialogoError getInstance() {
        if (INSTANCE == null) createInstance( null );
        return INSTANCE;
    }

	/**
	 * Constructor de la clase 
	 * @param owner
	 */
	private DialogoError( Frame owner ) {
		/** ventana padre, título del diálogo y que sea modal*/
		super( owner, s_tituloDialogo, true );
		
		this.setDefaultCloseOperation( DISPOSE_ON_CLOSE );
		
		/** colocar elementos del diálogo. */
		this.setLayout( new MigLayout ( "fill, inset unrel" ) );
		this.add( icono, "gap unrel, align center" );
		this.add( mensaje, "align center,  wrap, gap unrel" ); //alinear centro	
		
		/** añadir boton Aceptar */
		this.add( this.botAceptar, "gaptop unrel, span, align center");
		
		/** default button -> Aceptar */
		this.getRootPane().setDefaultButton( botAceptar );		
		
		/** ESC button -> Aceptar */
		KeyStroke esc = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
	    this.getRootPane( ).registerKeyboardAction( aclAceptar, "ACTION_CLOSE", esc, JComponent.WHEN_IN_FOCUSED_WINDOW);
		
	    /** Empaquetar y colocar */
		this.pack( );
		this.setLocationRelativeTo( owner );
		this.setResizable( false );
	}
	
	
	/** Función main para pruebas. */
	public static void main(String[] args) {
		javax.swing.JFrame v = new javax.swing.JFrame( s_tituloDialogo );
		javax.swing.JPanel p = new javax.swing.JPanel( );
		javax.swing.JButton b = new javax.swing.JButton(
				"Mostrar diálogo Error" );
		p.add( b );
		v.setContentPane( p );
		v.pack( );
		
		DialogoError.createInstance ( v );
		final DialogoError d = DialogoError.getInstance( );

		b.addActionListener( new ActionListener( ) {
			public void actionPerformed ( ActionEvent e ) {
				d.showErrorDialog( "Mensaje de error" );
				d.showErrorDialog( "Mensaje de error\n super largo de mil y pico páginas" +
						"donde yo me expando y me expndo como el universo" +
						"y nunca quedo saciado de espacio porque necesito explicarme" +
						"bien bien bien \n" +
						"Mensaje de error super largo de mil y pico páginas" +
						"donde yo me expando y me expndo como el universo" +
						"y nunca quedo \nsaciado de espacio porque necesito explicarme" +
						"bien bien bien " );
				d.showInfoDialog( "Hola que tal, ¿se autogestionará el tamaño?" );
				d.showInfoDialog( "s" );
				d.showInfoDialog( "Mensaje informativo" );

			}
		} );

		v.setDefaultCloseOperation( javax.swing.JFrame.EXIT_ON_CLOSE );
		v.setVisible( true );
	}
	
	
	public Object clone() throws CloneNotSupportedException {
        throw new CloneNotSupportedException(); 
	}

}

