/**
 * 
 */
package interfaz.grafico;

import java.awt.Color;
import java.util.LinkedList;

import javax.swing.event.ListDataEvent;


/**
 * @author angel
 *
 */
public class AbstractGraficoModel /*extends java.util.Observable */implements GraficoModel {

	private LinkedList <Double> datos;
	private LinkedList <String> nombres;
	private LinkedList <GraficoModelListener> listeners = new LinkedList<GraficoModelListener>();
	
	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#addGraficoModelListener(interfaz.grafico.GraficoModelListener)
	 */
	public void addGraficoModelListener(GraficoModelListener l) {
		listeners.add(l);
	}

	public AbstractGraficoModel (LinkedList<Double> datos, LinkedList <String> nombres) {
		
		this.datos=datos;
		this.nombres=nombres;

	}
	
	public AbstractGraficoModel () {
		
		this.datos = new LinkedList <Double>();
		this.nombres = new LinkedList <String>();

	}
	
	public void limpiar(){
		datos.clear();
		nombres.clear();
		fireDataChanged();
	}
	
	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getPorcionName(int)
	 */
	public String getPorcionName(int porcionIndex) {
		String resultado;
		if(porcionIndex >= 0 && porcionIndex < nombres.size() )
			resultado = nombres.get(porcionIndex);
		else
			resultado = null;
		return resultado;
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getSize()
	 */
	public int getSize() {
		
		return datos.size();
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getValueAt(int)
	 */
	public Double getValueAt(int porcionIndex) {
		Double resultado;
		if(porcionIndex >= 0 && porcionIndex < datos.size() )
			resultado = datos.get(porcionIndex);
		else
			resultado = null;
		return resultado;
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#isPorcionEditable(int)
	 */
	public boolean isPorcionEditable(int porcionIndex) {
		return false;
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#removeGraficoModelListener(interfaz.grafico.GraficoModelListener)
	 */
	public void removeGraficoModelListener(GraficoModelListener l) {
		listeners.remove(l);
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#setValueAt(java.lang.Object, java.lang.String, int)
	 */
	public void setValueAt(Double porcionValue, String porcionName,
			int porcionIndex) {
		nombres.add(porcionIndex, porcionName);
		datos.add(porcionIndex, (Double)porcionValue);

		fireDataChanged();
	}
	
	public void setValues (LinkedList<Double> datos, LinkedList <String> nombres) {
		
		this.datos=datos;
		this.nombres=nombres;
		
		fireDataChanged();
	}


	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getSum()
	 */
	public Double getSum() {
		Double resultado = 0.0;
		for(int i = 0; i < datos.size(); i++) {
			resultado+=(Double)datos.get(i);
		}
		
		return resultado;
	}
	
	protected void fireDataChanged() {
		ListDataEvent evento = new ListDataEvent(this, 0, 0, 0);//consultar ayuda
		
		int i;

		for (i=0; i<listeners.size(); i++)
		    ((GraficoModelListener)listeners.get(i)).graphicChanged(evento);

	}

	/* (non-Javadoc)fireDataChange
	 * @see interfaz.grafico.GraficoModel#addValue(java.lang.Object, java.lang.String)
	 */
	public void addValue(Double i, String s) {
		datos.add(i);
		nombres.add(s);
		
		fireDataChanged();		
	}

	
	public Color getPorcionColor( int i ){
		Color color=Color.WHITE;
		switch (i % 8){
			case 0: color = Color.BLUE; break;
			case 1: color = Color.GREEN; break;
			case 2: color = Color.MAGENTA; break;
			case 3: color = Color.ORANGE; break;
			case 4: color = Color.CYAN; break;
			case 5: color = Color.PINK; break;
			case 6: color = Color.YELLOW; break;
			case 7: color = Color.RED; break;
			default: break;
		}
		return color;
	}
}
