/**
 * 
 */
package interfaz.vcompilacion;

import java.awt.Component;
import java.awt.Frame;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Vector;

import interfaz.Vista;
import interfaz.acciones.CrearYGestionarAcciones;
import interfaz.acciones.InterfazCambiarDirectorioTrabajo;
import interfaz.acciones.InterfazCambiarNombre;
import interfaz.acciones.InterfazCompilar;
import interfaz.acciones.InterfazCrearCompilacion;
import interfaz.dialogos.dialogoAbrir.DialogoAbrir;
import interfaz.dialogos.dialogoCambiarComando.DialogoCambiarComando;
import interfaz.dialogos.dialogoDirectorio.JDirectoryChooser;
import interfaz.dialogos.error.DialogoError;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import modelo.vcompilacion.ModeloCompilacion;
import modelo.vedicion.ModeloEdicion;
import negocio.PreferenciasSingleton;
import negocio.guigprof;
import negocio.ejecutarOrdenes.EjecutorComandos;
import net.miginfocom.swing.MigLayout;

/**
 * @author angel2
 */
public class VistaCompilacion extends JPanel implements Vista, InterfazCrearCompilacion,
	InterfazCambiarNombre,	InterfazCompilar, InterfazCambiarDirectorioTrabajo {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7934067204858477027L;

	/**
	 * Esta es la clase principal, ventana principal del interfaz o gui.
	 */
	private Frame gui;

	/** recursos para dar soporte sobre idiomas	 */
	static ResourceBundle idiomas = ResourceBundle.getBundle("interfaz.idiomas.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	
	/** recursos de iconos etc... */
	static ResourceBundle recursos = ResourceBundle.getBundle("interfaz.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	
	/** recursos de idioma para los errores */
	static ResourceBundle errores = ResourceBundle.getBundle("interfaz.errores.Errores", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	
	/** dialogo Abrir **/ 
	DialogoAbrir					dialogoAbrir		= new DialogoAbrir( );

	/** barra de tareas específica de esta vista.	 */
	JToolBar barraDeTareas = new JToolBar();
	
	/**********Constructor PANEL Y GESTION PANEL***************/
	
	/**
	 * @param guIguigprof
	 */
	public VistaCompilacion( Frame gui) {
		super(new MigLayout("fill"));
		this.gui = gui;
		idiomas = ResourceBundle.getBundle("interfaz.idiomas.Recursos", PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias().getLocale());
		crearPanel();
		crearBarraTareas();
	}

	/** Panel general de la vista compilación */
//	private JPanel panel;
	/** Panel de pestañas inferior, donde se muestra la salida de la compilación */
	private JTabbedPane	pestañas	= new JTabbedPane( );
	{
		ChangeListener cambioPestaña = new ChangeListener () {

			@Override
			public void stateChanged ( ChangeEvent arg0 ) {
				activarActions(visible);
			}
			
		};
		pestañas.addChangeListener( cambioPestaña );
	}

	protected void crearBarraTareas() {
		JButton botAbrirCompilacion = new JButton ();
		{
			botAbrirCompilacion.setHideActionText(true);
			botAbrirCompilacion.setAction(CrearYGestionarAcciones.getInstance().getAccionCrearCompilacion( ) );
		}
		JButton botNombreEjecutable = new JButton ();
		{
			botNombreEjecutable.setHideActionText(true);
			botNombreEjecutable.setAction(CrearYGestionarAcciones.getInstance().getAccionNombreEjecutable( ) );
		}
		JButton botCerrar = new JButton ();
		{
			botCerrar.setHideActionText(true);
			botCerrar.setAction(CrearYGestionarAcciones.getInstance().getAccionCerrar());
		}
		JButton botEjecutarCompilado = new JButton ();
		{
			botEjecutarCompilado.setHideActionText(true);
			botEjecutarCompilado.setAction(CrearYGestionarAcciones.getInstance().getAccionEjecutarCompilado());
		}
		JButton botCompilar = new JButton ();
		{
			botCompilar.setHideActionText(true);
			botCompilar.setAction(CrearYGestionarAcciones.getInstance().getAccionCompilar());
		}
		{
			barraDeTareas.setAutoscrolls(true);
			barraDeTareas.setFloatable(false);
			barraDeTareas.add(botAbrirCompilacion);
			barraDeTareas.add(botCerrar);
			barraDeTareas.addSeparator( );
			barraDeTareas.add(botNombreEjecutable);
			barraDeTareas.add(botCompilar);
			barraDeTareas.addSeparator( );
			barraDeTareas.add(botEjecutarCompilado);
		}
	}

	/**
	 * @return
	 */
	public JToolBar getBarraDeTareas() {
		return barraDeTareas;
	}
	
	protected void crearPanel () {
		/*Aquí hay que hacer el panel*/
		setOpaque(true);

		/* Se añade las pestañas. */
		add(pestañas, "grow");
		
		/* A continuación, se crea un panel inicial vacío. */
		pestañas.addTab( "" , new JPanelCompilacion( null ) );
	}
	
	protected void añadirPestaña ( ModeloCompilacion mo ) {
		/** nombre futura compilacion */
		if ( mo.getNombrePrograma( ) == null ) {
			String nombrePrograma;
			nombrePrograma = idiomas.getString( "nombreEstandarCompilacion" );
			mo.setNombrePrograma( nombrePrograma );
		}
		String nombrePrograma = mo.getNombrePrograma( );
		
		Icon icon = (Icon) recursos.getObject( "icn_Compilando" );
		
		JPanelCompilacion pestañaNueva = new JPanelCompilacion( mo );

		/** Crear la pestaña correctamente */
		pestañas.addTab(nombrePrograma, pestañaNueva);
		int index = pestañas.indexOfComponent(pestañaNueva);
		
		javax.swing.JLabel tabRenderer = new javax.swing.JLabel ( nombrePrograma );
		tabRenderer.setIcon(icon);
		tabRenderer.setIconTextGap(5);
		tabRenderer.setHorizontalTextPosition(SwingConstants.RIGHT);
		//tabRenderer.setToolTipText(camino);
				
		pestañas.setTabComponentAt(index, tabRenderer);
		pestañas.setToolTipTextAt(index, nombrePrograma);
		
		pestañas.setSelectedComponent(pestañaNueva);
	}
	/*********** métodos *************/

	public void crearCompilacion ( ) {

		String nombre_ejecutable = this.getNuevoNombre( );
		File dir = PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getWorkspaceCompilacion( );
		DialogoNuevaCompilacion dialog = new DialogoNuevaCompilacion( guigprof.getInstance( ).getGUI( ), nombre_ejecutable, dir );
		if( dialog.showDialog( ) == DialogoNuevaCompilacion.OK_OPTION ) {
			ModeloCompilacion mo = new ModeloCompilacion();
			if( dialog.getNombreEjecutable( ).length( ) > 0 ) {
				nombre_ejecutable = dialog.getNombreEjecutable( );
			}
			mo.setNombrePrograma( nombre_ejecutable );
			if( dialog.getDirectorioTrabajo( ) != null ) {
				dir = dialog.getDirectorioTrabajo( );
			}
			mo.setDirectorioTrabajo( dir );
			abrirNuevaCompilacion( mo );
		} else {
		}

		
	}
	
	public void abrirNuevaCompilacion ( ModeloCompilacion mo ) {

		if ( mo != null ) {
			/** intentando crear una pestaña compilacion */
			
			if( pestañas.getTabCount( ) == 1 && pestañas.getComponentAt( 0 ) instanceof JPanelCompilacion && ( (JPanelCompilacion) pestañas.getComponentAt( 0 )).isVacio( ) ) {
				pestañas.removeAll( );
				añadirPestaña(mo);
				
			} else {
				añadirPestaña(mo);
			}
		}
	}
	
	public void compilarFuente ( ModeloEdicion me ) {
		if ( me != null ) {
			File fichero = me.getFichero( );
			if ( fichero != null ) {
				ModeloCompilacion mo = new ModeloCompilacion( );
				String nombreMódulo = fichero.getName( );
				int i = nombreMódulo.lastIndexOf( "." );
				if ( i > 0 )
					nombreMódulo = nombreMódulo.substring( 0 , i );
				mo.setNombrePrograma( nombreMódulo );
				mo.setDirectorioTrabajo( fichero.getParentFile( ) );
				mo.addFichero( fichero );
				this.abrirNuevaCompilacion( mo );
			}
		}
	}
	
	protected String getNuevoNombre( ) {
		return idiomas.getString( "nombreEstandarCompilacion" ) + "_" + ( pestañas.getTabCount( ) );
	}
	
	public void cerrarPestañaActiva () {
		//		Programar un trabajo para el hilo de despacho
		final int indicePestañaActiva = pestañas.getSelectedIndex();
		if (indicePestañaActiva != -1) {
			//usar el propiedades para estos comentarios...
			String cuestión= new String(idiomas.getString("eti_compilacion_seguroCerrar"));
			String título= new String(idiomas.getString("eti_cerrar"));
			int está_usted_seguro= JOptionPane.showConfirmDialog(gui,
					cuestión,
					título,
					JOptionPane.YES_NO_OPTION);

			if(está_usted_seguro==JOptionPane.YES_OPTION){
				if( pestañas.getTabCount( ) > 1 ) {
					pestañas.removeTabAt(indicePestañaActiva);
					
				} else if( pestañas.getTabCount() == 1 ) {
					pestañas.removeAll( );
					pestañas.add( new JPanelCompilacion( ) );
					
				}
			}
		}
	}
	
	public void cerrarTodo ( ) {
		pestañas.removeAll( );
	}
	
	public void añadirFicheros( ) {
		ModeloCompilacion mc = this.getModeloCompilacionActivo( );
		if ( mc != null ) {
				DialogoAbrir dialogoAbrir = new DialogoAbrir();
		
				// elegir ficheros
				File [] ficheros = null;
				/** Mostrar el dialogo de abrir (DialogoAbrir). */
				dialogoAbrir.setCurrentDirectory( mc.getDirectorioTrabajo( ) );
				if ( dialogoAbrir.showOpenDialogEdicion( gui, true ) == JFileChooser.APPROVE_OPTION ) {
					// *podría hacerlo para múltiples ficheros seleccionados...
					ficheros = dialogoAbrir.getSelectedFiles( );

					// añadir ficheros
					for ( int i = 0 ; i < ficheros.length ; i++ )
						mc.addFichero( ficheros[i] );
				}
		}		
	}

	public void quitarFicheros() {
		JPanelCompilacion pc = getPanelActivo();
		if ( pc != null ) {
			Object[] seleccionados = pc.getFicherosSeleccionados( );
			ModeloCompilacion mc = pc.getModel( );
			if ( mc != null ) {
				for ( int i = 0 ; i < seleccionados.length ; i++ )
					mc.removeFichero( seleccionados[i] );
			}
		}
	}

	/**
	 * Método para cambiar el directorio de trabajo en la pestaña activa.
	 */
	public void cambiarDirectorioTrabajo ( ) {
		ModeloCompilacion mo = this.getModeloCompilacionActivo( );
		if ( mo != null ) {
			JDirectoryChooser dialogo = new JDirectoryChooser( new File( "/" ) );
			File directorio_inicial = mo.getDirectorioTrabajo( );
			if ( directorio_inicial != null ) {
				dialogo.setSelectedDirectory( directorio_inicial );
			}
			if ( dialogo.showDialog( gui ) == JDirectoryChooser.OK_OPTION ) {
				File directorio_final = dialogo.getSelectedDirectory( );
				if ( directorio_final != null && directorio_final.isDirectory( ) ) {
					mo.setDirectorioTrabajo( dialogo.getSelectedDirectory( ) );
				}
			}

		}
	}
	
	/**
	 * Método para editar el nombre del programa 
	 */
	public void cambiarNombre ( ) {
		/** editarNombrePrograma */
		JPanelCompilacion pc = getPanelActivo();
		if ( pc != null ) {
				ModeloCompilacion mo = pc.getModel( );
				
				if( mo != null ) {
					DialogoCambiarNombre dialogo = new DialogoCambiarNombre( mo.getNombrePrograma( ) );
					if( dialogo.showDialog( gui ) == DialogoCambiarNombre.OK_OPTION ) {
						String nombre = dialogo.getNombre();
						mo.setNombrePrograma( nombre );
						
						int index = pestañas.indexOfComponent( pc );
						
						pestañas.setTitleAt( index , nombre );
						( (javax.swing.JLabel) pestañas.getTabComponentAt( index ) ).setText(nombre);
						pestañas.setToolTipText( nombre );
					}
				}
		}
	}
	
	/**
	 * Método para editar el comando de compilación 
	 */
	public void editarComando ( ) {

		JPanelCompilacion pg = getPanelActivo();
		if ( pg != null ) {
				ModeloCompilacion mo = pg.getModel( );
				DialogoCambiarComando diaOrdenes = new DialogoCambiarComando( guigprof.getInstance( ).getGUI( ) );
				diaOrdenes.setComando( mo.getCompilador( ) );

				boolean ordenModificada = diaOrdenes.showDialog( );
				
				Vector <String> orden = diaOrdenes.getComando();
				
				if ( ordenModificada && orden != null && orden.size( ) > 0 ) {
					
					mo.setCompilador( orden );
					
				}

		}
	}
	
	/**
	 * Método para compilar los ficheros elegidos en la pestaña activa.
	 */
	@Override
	public void compilar ( ) {
		JPanelCompilacion pc = getPanelActivo( );
		if ( pc != null ) {
			ModeloCompilacion mo = pc.getModel( );
			if ( mo != null ) {
				final File dirTrabajo = mo.getDirectorioTrabajo( );
				/*final ArrayList < File > ficheros = mo.getListaFicheros( );
				final String nombrePrograma = mo.getNombrePrograma( );*/
				final String [] ordenes = mo.getComandoCompilacion( );

				try {
					/** realizar compilación*/
					final Thread compilador = new EjecutorComandos( ordenes , dirTrabajo , pc );
					
					compilador.start( );
					
					/** hilo para actualizar botones cuando acabe la ejecución de compilación */
					new Thread() {
						public void run () {
							try {
								compilador.join();
								
								activarActions(visible);
								
							} catch ( InterruptedException e ) {
								DialogoError.getInstance( ).showErrorDialog( 
										errores.getString("err_errorCompilacion") );
							}
							
						}
					}.start( );
				} catch ( Exception e ) {
					DialogoError.getInstance( ).showErrorDialog( 
							errores.getString("err_errorCompilacion") );
				}
			}
		}
	}
	
/*	public void cambiarFuente ( Font f ) {
		for (int i = 0; i < pestañas.getTabCount( ); i ++ ) {
			Component c = pestañas.getComponentAt( i );
			if( c instanceof JPanelCompilacion ) {
				( ( JPanelCompilacion ) c  ).cambiarFuente( f );
			}
		}
	}*/
	
	/******************* pestañas ***********************/
	
	public Vector < ModeloCompilacion > getModelos () {
		Vector < ModeloCompilacion > vModelos = new Vector < ModeloCompilacion >();
		
		for (int i = 0; i < pestañas.getTabCount( ); i ++ ) {
			Component c = pestañas.getComponentAt( i );
			if( c instanceof JPanelCompilacion ) {
				ModeloCompilacion moC = ( ( JPanelCompilacion ) c ).getModel( );
				if ( moC != null )
					vModelos.add( moC );
			}
		}
		
		return vModelos;
	}
	
	/** devuelve el modelo compilación del panel activo. Null si no hay ninguno; */
	public ModeloCompilacion getModeloCompilacionActivo() {
		ModeloCompilacion mo = null;
		JPanelCompilacion pc = getPanelActivo();
		if ( pc != null ) {
			mo = pc.getModel( );
		}
		return mo;
	}
	
	/** devuelve el panel activo. Null si no hay ninguno; */
	private JPanelCompilacion getPanelActivo () {
		JPanelCompilacion pc = null;
		if ( pestañas.getComponentCount() > 0 ) {
			Component c = pestañas.getSelectedComponent();
			if ( c instanceof JPanelCompilacion ) {
				pc = (JPanelCompilacion) c;
			}
		}
		return pc;
	}
	/****************Visibilidad********************/
	boolean visible = true;
	
	public void setActivo(boolean b){
		visible = b;
		setVisible(b);
		//también hay que anular las acciones que no correspondan.
		activarActions ( b );
		
		barraDeTareas.setEnabled(b);
		barraDeTareas.setVisible(b);
	}
	public boolean isActivo () {
		return visible;
	}
	public void activarActions ( boolean vistaActiva ) {
		boolean panelNoVacio = false;
		boolean siguientePaso = false;
		if ( vistaActiva && this.getPanelActivo( ) != null && !this.getPanelActivo( )
				.isVacio( ) ) {
			panelNoVacio = true;
		}
		if ( panelNoVacio == true && this.getPanelActivo( ).getModel( ).isCompiled( ) ) {
			siguientePaso = true;
		}
		
		CrearYGestionarAcciones a = CrearYGestionarAcciones.getInstance( );
		
		a.activarActionsCompilacion( vistaActiva, panelNoVacio , siguientePaso );
		
		/*boolean panelNoVacio = false;
		boolean siguientePaso = false;
		if ( vistaActiva && this.getPanelActivo( ) != null && !this.getPanelActivo( )
				.isVacio( ) ) {
			panelNoVacio = true;
		}
		CrearYGestionarAcciones.getInstance().getAccionCompilar().setEnabled( panelNoVacio );
		CrearYGestionarAcciones.getInstance().getAccionEditarComandoCompilacion( ).setEnabled( panelNoVacio );
		CrearYGestionarAcciones.getInstance().getAccionCambiarDirectorioTrabajoCompilacion( ).setEnabled( panelNoVacio );
		CrearYGestionarAcciones.getInstance().getAccionQuitarFicheros( ).setEnabled( panelNoVacio );
		CrearYGestionarAcciones.getInstance().getAccionAñadirFicheros( ).setEnabled( panelNoVacio );
		CrearYGestionarAcciones.getInstance().getAccionNombreEjecutable( ).setEnabled( panelNoVacio );
		CrearYGestionarAcciones.getInstance().getAccionCerrar( ).setEnabled( panelNoVacio );
		
		if ( panelNoVacio == true && this.getPanelActivo( ).getModel( ).isCompiled( ) ) {
			siguientePaso = true;
		}
		CrearYGestionarAcciones.getInstance().getAccionEjecutarCompilado( ).setEnabled( siguientePaso );
		if ( vistaActiva == true ) {
			CrearYGestionarAcciones.getInstance().getAccionVistaCompilacion( ).putValue( Action.LARGE_ICON_KEY , recursos
						.getObject( "img_vistaCompilacion_activado" ) );
		} else {
			CrearYGestionarAcciones.getInstance().getAccionVistaCompilacion( ).putValue( Action.LARGE_ICON_KEY , recursos
					.getObject( "img_vistaCompilacion" ) );
		}*/
	}

}
