/**
 * Clase JPanelEdicion.java es una aplicación Java 6.
 * Clase que especializa un panel (JPanel). En este panel se muestra
 *  la salida de la ejecución de un programa.
 * Usa un objeto ILineaComando para la ejecución del programa. 
 * Pertenece a la aplicación guigprof para el proyecto fin de carrera. 
 * Autor: Ángel García Fernández.
 * Tutor: Francisco Palomo Lozano e Inmaculada Medina Bulo
 * Fecha: Agosto de 2007.
 */
package interfaz.vedicion;


import interfaz.dialogos.error.DialogoError;

import java.awt.Font;
import java.io.File;
import java.util.ResourceBundle;

//import javax.swing.JButton;
import javax.swing.BorderFactory;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import modelo.vedicion.ModeloEdicion;
import negocio.PreferenciasSingleton;
import negocio.ejecutarOrdenes.InterfazOrdenes;
import net.miginfocom.swing.MigLayout;


/**
 * @author angel
 *
 */
public class JPanelEdicion extends JPanel implements ChangeListener, InterfazOrdenes {

	private static final long	serialVersionUID	= -7191529819222353003L;
	
	/** recursos de idiomas para errores */
	static ResourceBundle errores = ResourceBundle.getBundle("interfaz.errores.Errores", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
/************************PROPIEDADES***********************/	
	
	/**
	 * Área de texto donde se muestra el contenido del fichero fuente.
	 * @see fichero
	 */
	private JTextArea atxTexto = new JTextArea();
//	/**
//	* Borde con título del área de texto. Muestra el path del fichero fuente que se esté mostrando.
//	* @see atxTexto
//	*/
	private JLabel bordeTitulo = new JLabel("");
	
	/************************ CONTRUCTOR ***********************/
	private ModeloEdicion	moEdicion = null;
	
	public JPanelEdicion () {
		super(new MigLayout("fill"));
		
		crearComponentes();
		this.setModel( null );
	}	
	/**
	 * @param contenido
	 */
	public JPanelEdicion ( ModeloEdicion me ) {
		super(new MigLayout("fill"));
		
		crearComponentes();
		this.setModel( me );
	}
	
	/************************ METODOS ***********************/
	public void cambiarFuente ( Font f ) {
		this.atxTexto.setFont( f );		
	}
		
	public boolean isVacio () {
		return this.moEdicion == null;
	}
	
	protected void setModel ( ModeloEdicion me ) {
		this.moEdicion = me;
		
		if ( me != null ) {
			this.moEdicion.addChangeListener( this );
		}
		actualizarModel();
	}
	
	public ModeloEdicion getModel () {
		return this.moEdicion;
	}
	
	

	protected void crearComponentes () {
		setBorder(new EtchedBorder());
		setOpaque(true);
		
		//aplicar el tipo de letra
		//atxTexto.setFont(new Font("courier", Font.PLAIN, 12));
		atxTexto.setFont( PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getEstadisticasFont( ) );

		//a partir de aquí es meter el contenido del fichero en el area de texto (atxTexto)

		//atxTexto.setLineWrap(true); dá problemas al usar setText ¿?¿?
		atxTexto.setEditable(false);
		atxTexto.setWrapStyleWord(true);

		//El scroll del atxTexto
		JScrollPane areaScrollPane = new JScrollPane(atxTexto);
		areaScrollPane.setVerticalScrollBarPolicy(
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		// no areaScrollPane.setPreferredSize(new Dimension(250, 250));
		areaScrollPane.setBorder(
				BorderFactory.createCompoundBorder(
								BorderFactory.createEmptyBorder(5,5,5,5),
								areaScrollPane.getBorder()));

		//auto-añadir el area de texto con scroll al panel
		add(bordeTitulo,"wrap");
		add(areaScrollPane,"grow");
	}
	
	@Override
	public void stateChanged ( ChangeEvent e ) {
		if ( e.getSource( ) instanceof ModeloEdicion ) {
			this.actualizarModel( );
		}
		
	}

	private void actualizarModel ( ) {

		if ( this.moEdicion != null ) {
	
			/* atxTexto */
			atxTexto.setText( this.moEdicion.getTexto() );

			/* brTitulo */
			File f = this.moEdicion.getFichero( );
			bordeTitulo.setText( (f != null? f.getName( ):"" ) + ":" );
		}
		else {
			
			/* atxTexto */
			atxTexto.setText( "" );

			/* brTitulo */
			bordeTitulo.setText( "" );
		}

	}

	@Override
	public void empezarOrden ( String comando ) {
		//Nada que hacer
	}

	@Override
	public void ordenErrorIndeterminado( String comando, File directorio, Exception e ){
		String saltoLinea = "\n";
		String descripcion;

		descripcion = errores.getString( "err_errorEjecuciónOrden" )
				+ saltoLinea + errores.getString( "err_errorEjecuciónOrden_orden" )
				+ saltoLinea + comando
				+ saltoLinea + errores.getString( "err_errorEjecución_directorio" )
				+ saltoLinea + directorio.getName( )
				+ saltoLinea
				+ saltoLinea + errores.getString( "err_detalles:" )
				+ saltoLinea + e.getMessage( );
		
		DialogoError.getInstance( ).showErrorDialog( descripcion );
	}

	
	@Override
	public void terminarOrden ( int resultado , String estandar , String errores ) {
		moEdicion.actualizar( );		
	}

}
