package interfaz.vestadisticas;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;

import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.UIManager;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.BadLocationException;
import javax.swing.text.JTextComponent;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import net.miginfocom.swing.MigLayout;

public class CabeceraMultilineaTableCellRenderer  extends JPanel implements TableCellRenderer {

	/** UID SERIAL VERSION*/
	private static final long	serialVersionUID	= 5679658297447485908L;
	
	static private int AlturaMáxima = 80;
	
	JTextPane cabecera = new JTextPane();
	
	public CabeceraMultilineaTableCellRenderer() {
		super( new MigLayout("fill, insets 0"));
		
		cabecera.setBorder( UIManager.getBorder( "TableHeader.cellBorder" ) );
		
		cabecera.setOpaque( true );
		cabecera.setFocusable( true );
		cabecera.setForeground( UIManager.getColor( "TableHeader.foreground" ) );
		cabecera.setFont( UIManager.getFont( "TableHeader.font" ) );
		//cabecera.setPreferredSize( new Dimension (1,1) );
		cabecera.setMinimumSize( new Dimension (1,1) );
		//cabecera.setHorizontalTextPosition( JButton.CENTER );
		
		this.add( cabecera, "grow" );
	}	
	
	@Override
	public Component getTableCellRendererComponent ( JTable tabla ,
			Object valor , boolean seleccionado , boolean tieneElFoco , int fila ,
			int columna  ) {
		String cadena = ( valor == null ) ? "" : valor.toString( );

		cabecera.setText( cadena );
		if( tieneElFoco )
			cabecera.setBackground( UIManager.getColor( "TebleHeader.focusCellBackground" ) );
		else
			cabecera.setBackground( UIManager.getColor( "TebleHeader.background" ) );
		
		/** centrar texto */
		StyledDocument doc = cabecera.getStyledDocument();
		SimpleAttributeSet center = new SimpleAttributeSet();
		StyleConstants.setAlignment(center, StyleConstants.ALIGN_CENTER);
		doc.setParagraphAttributes(0, doc.getLength(), center, false);
		/** correcto tamaño */
		int ancho = tabla.getColumnModel( ).getColumn( columna ).getWidth( );
		CabeceraMultilineaTableCellRenderer.setCorrectDimensionToJTextComponent( cabecera,
				ancho);
		
		this.setToolTipText( cadena );
		
		tabla.revalidate( );
		
		return this;
	}
	
	private static void setCorrectDimensionToJTextComponent ( JTextComponent x, int ancho) {
		
        int height = AlturaMáxima;
        int width = ancho;
        
		Dimension minimalSize = new Dimension ( width, height );
		
		x.setPreferredSize(minimalSize);
		x.setSize(minimalSize);            
        
		Rectangle r_pos = new Rectangle ( );

		try {
			int length = x.getDocument( ).getLength( );
			
			r_pos = x.modelToView( length );
				
			height = r_pos.y + r_pos.height;
			
			if ( height > AlturaMáxima ) {
				height = AlturaMáxima;
			}
							
			Dimension actualSize = new Dimension ( width, height );
			x.setPreferredSize( actualSize);
			x.setSize( actualSize );
			
		} catch ( BadLocationException e ) {
			e.printStackTrace();
		}
	}
}

