/**
 * 
 */
package modelo.ficheros;

import java.io.File;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;


/**
 * @author angel2
 * 
 */
public class UtilidadesExtensionFichero {

	static ResourceBundle	propiedadesImagenes	= ResourceBundle
														.getBundle( "interfaz.Recursos" );


	public static TipoFichero getTipo ( File f ) {
		TipoFichero tipo = TipoFichero.DESCONOCIDO;

		String extension = getExtension( f );

		if ( isC( extension ) ) {
			tipo = TipoFichero.C;
		} else if ( isCpp( extension ) ) {
			tipo = TipoFichero.CPP;
		} else if ( isPascal( extension ) ) {
			tipo = TipoFichero.PASCAL;
		} else if ( isFortran( extension ) ) {
			tipo = TipoFichero.FORTRAN;
		} else if ( isGprof( extension ) ) {
			tipo = TipoFichero.GPROF;
		} else if ( isGprofXML( f ) ) {
			tipo = TipoFichero.GPROFXML;
		} else if ( isSesion( f ) ) {
			tipo = TipoFichero.SESION;
		} else if ( isArchivoDatosGprof( f ) ) {
			tipo = TipoFichero.DATAFILE;
		}

		return tipo;
	}

	/*
	 * Coge la extensión de un fichero y la devuelve Si no tiene, devuelve una
	 * cadena vacía
	 */
	public static String getExtension ( File f ) {
		String extension = "";
		String s = f.getName( );
		int i = s.lastIndexOf( '.' );

		if ( i > 0 && i < s.length( ) - 1 ) {
			extension = s.substring( i + 1 ).toLowerCase( );
		}
		return extension;
	}

	/**
	 * Deduce el tipo del fichero a partir del su nombre, mediante su extensión.
	 * 
	 * @return Devuelve el tipo del fichero según su extensión. Se pueden
	 *         distinguir 4 tipos enumerados: Desconocido, C, Pascal o Fortran.
	 * @see Desconocido
	 * @see C
	 * @see Pascal
	 * @see Fortran
	 */
	// ahora mira directamente en la variable fichero.getName(), en vez de por
	// parámetro...
	// * @param nombrefichero Es un <code>String</code> que contiene un nombre
	// de fichero.
	public static boolean isC ( File f ) {
		String extension = new String( getExtension( f ) );
		boolean resultado;

		resultado = isC( extension );

		return resultado;
	}

	public static boolean isCpp ( File f ) {
		String extension = new String( getExtension( f ) );
		boolean resultado;

		resultado = isCpp( extension );

		return resultado;
	}
	
	
	public static boolean isPascal ( File f ) {
		String extension = new String( getExtension( f ) );
		boolean resultado = isPascal( extension );
		return resultado;
	}

	public static boolean isFortran ( File f ) {
		String extension = new String( getExtension( f ) );
		boolean resultado = isFortran ( extension );
		return resultado;
	}

	public static boolean isGprof ( File f ) {
		String extension = new String( getExtension( f ) );
		boolean resultado = isGprof( extension );

		return resultado;
	}

	public static boolean isLenguajes ( File f ) {
		boolean resultado;

		if ( isC( f ) || isPascal( f ) || isFortran( f ) 
				|| isCpp( f ) )
			resultado = true;
		else
			resultado = false;

		return resultado;
	}

	public static boolean isXML ( File f ) {
		return f.getName( ).endsWith( ".xml" );
	}

	public static boolean isGprofXML ( File f ) {
		return f.getName( ).endsWith( ".gprof.xml" );
	}

	public static boolean isArchivoDatosBinarios ( File f ) {
		return f.getName( ).endsWith( ".out" );
	}
	
	public static boolean isArchivoDatosGprof ( File f ) {
		return f.getName( ).endsWith( ".gmon.out" );
	}
	
	public static boolean isSesion ( File f ) {
		return f.getName( ).endsWith( ".session" );
	}

	public static ImageIcon getIcono ( File f ) {
		ImageIcon icono = null;
		if ( !f.isDirectory( ) ) {
			if ( isC( f ) ) {
				icono = (ImageIcon) propiedadesImagenes.getObject( "icn_C" );
			} else if ( isPascal( f ) ) {
				icono = (ImageIcon) propiedadesImagenes
						.getObject( "icn_Pascal" );
			} else if ( isFortran( f ) ) {
				icono = (ImageIcon) propiedadesImagenes
						.getObject( "icn_Fortran" );
			} else if ( isGprof( f ) ) {
				icono = (ImageIcon) propiedadesImagenes.getObject( "icn_Gprof" );
			} else if ( isCpp( f )) {
				icono = (ImageIcon) propiedadesImagenes.getObject( "icn_Cpp" );
			} else if ( isSesion( f ) ) {
				icono = (ImageIcon) propiedadesImagenes.getObject( "icn_Sesion" );
			} else if ( isArchivoDatosBinarios( f ) ) {
				icono = (ImageIcon) propiedadesImagenes.getObject( "icn_FDatos" );
			} else if ( isGprofXML( f ) ) {
				icono = (ImageIcon) propiedadesImagenes.getObject( "icn_GprofXML" );
			} else if ( UtilidadesExtensionFichero.isXML( f ) ) {
				icono = (ImageIcon) propiedadesImagenes.getObject( "icn_XML" );
			} else if ( f.canExecute( ) ) {
				icono = (ImageIcon) propiedadesImagenes.getObject( "icn_Programa" );
			} else icono = (ImageIcon) propiedadesImagenes.getObject("icn_Desconocido");
			// else 
				//icono = null;
		}
		return icono;
	}

	private static boolean isC ( String extension ) {
		boolean resultado = false;

		extension.toLowerCase( );
		
		if ( extension.equals( "c" ) /*|| extension.equals( "cpp" )*/
				|| extension.equals( "cc" ) /*|| extension.equals( "c++" )*/
				|| extension.equals( "cp" ) /*|| extension.equals( "cxx" )*/ )
			resultado = true;
		return resultado;
	}

	private static boolean isPascal ( String extension ) {
		boolean resultado = false;

		extension.toLowerCase( );
		
		if ( extension.equals( "pas" ) )
			resultado = true;
		return resultado;
	}

	private static boolean isCpp ( String extension ) {
		boolean resultado = false;

		extension.toLowerCase( );
		
		if ( extension.equals( "cpp" ) || extension.equals( "c++" ) 
				|| extension.equals( "cxx") )
			resultado = true;
		return resultado;
	}

	private static boolean isFortran ( String extension ) {
		boolean resultado = false;
		
		extension.toLowerCase( );

		if ( extension.equals( "f" ) || extension.equals( "f90" )
				|| extension.equals( "for" ) || extension.equals( "f95" )
				|| extension.equals( "fpp") )
			resultado = true;
		else
			resultado = false;

		return resultado;
	}

	private static boolean isGprof ( String extension ) {
		boolean resultado = false;
		
		extension.toLowerCase( );
		
		if ( extension.equals( "gprof" ) )
			resultado = true;
		return resultado;
	}

	public static File addExtension ( TipoFichero tipo , File fichero ) {
		String extension = new String();
		File resultado = fichero;

		switch ( tipo ) {
		case 		C		:	 extension = ".c"; break;
		case 		CPP		:	 extension = ".cpp"; break;
		case 		PASCAL	:	 extension = ".pas"; break;
		case 		FORTRAN :	 extension = ".f"; break;
		case 		DATAFILE :	 extension = ".gmon.out"; break;
		case 		GPROF	:	 extension = ".gprof"; break;
		case 		GPROFXML	:	 extension = ".gprof.xml"; break;
		case 		SESION 	:	 extension = ".session"; break;
		}

		if ( !extension.equals( "" ) ) {
			resultado = new File ( fichero.getAbsolutePath( ) + extension );
		}
		return resultado;
	}

}
