package modelo.vejecucion;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

import javax.swing.DefaultListModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import negocio.PreferenciasSingleton;

public class ModeloEjecucion {
	private DefaultListModel	molstEntradas	= new DefaultListModel( );
	
	private DefaultListModel	molstArgumentos	= new DefaultListModel( );

	private class ControladorModeloLista implements ListDataListener {
		@Override
		public void intervalAdded ( ListDataEvent arg0 ) {
			OnChange( );
		}

		@Override
		public void intervalRemoved ( ListDataEvent arg0 ) {
			OnChange( );
		}

		@Override
		public void contentsChanged ( ListDataEvent arg0 ) {
			OnChange( );
		}
	}

	{
		molstEntradas.addListDataListener( new ControladorModeloLista( ) );
		molstArgumentos.addListDataListener( new ControladorModeloLista( ) );
	}

	private File					directorioTrabajo	= PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getWorkspaceEjecucion( );

	private File					ficheroResultados	= null;
	
	public static String extension = ".gmon.out";


	private String					nombreResultadoEjecucion		= null;

	private File					programa			= null;

	ArrayList < ChangeListener >	listeners			= new ArrayList < ChangeListener >( 0 ); // listeners

	
	public void addChangeListener ( ChangeListener listener ) {
		listeners.add( listener );
	}

	public void removeChangeListener ( ChangeListener listener ) {
		listeners.remove( listener );
	}

	private void OnChange ( ) {
		ChangeEvent e = new ChangeEvent( this );
		Iterator < ChangeListener > it = listeners.iterator( );
		while ( it.hasNext( ) ) {
			it.next( ).stateChanged( e );
		}
	}

	public DefaultListModel getEntradasListModel ( ) {
		return molstEntradas;
	}
	
	public DefaultListModel getArgumentosListModel ( ) {
		return molstArgumentos;
	}

	@SuppressWarnings ( "unchecked" )
	public ArrayList < String > getListaEntradas ( ) {
		ArrayList < String > entradas = (ArrayList < String >) Collections
				.list( molstEntradas.elements( ) );
		return entradas;
	}
	
	@SuppressWarnings ( "unchecked" )
	public ArrayList < String > getListaArgumentos ( ) {
		ArrayList < String > argumentos = (ArrayList < String >) Collections
		.list( molstArgumentos.elements( ) );
		return argumentos;
	}
	
	public File getDirectorioTrabajo ( ) {
		return directorioTrabajo;
	}

	public void setDirectorioTrabajo ( File dir ) {
		this.directorioTrabajo = dir;
		OnChange( );
	}

	public String getNombreResultadosEjecucion ( ) {
		return this.nombreResultadoEjecucion;
	}

	public void setNombreResultadosEjecucion ( String nombre ) {
		if ( nombre != null && nombre.compareTo( "" ) != 0 ) {
			if( !nombre.endsWith( extension ) ) {
				nombre += extension;
			}
		}
		else 
			nombre = null;
		
		this.nombreResultadoEjecucion = nombre;
		OnChange( );
	}

	public File getPrograma () {
		return programa;
	}
	
	public void setPrograma( File prog ) {
		this.setNombreResultadosEjecucion( prog.getName( ) );
		this.programa = prog;
		OnChange();
	}
	

	public String getComandoEjecucion ( ) {
		String resultado = new String();
		
		if ( this.getPrograma( ) != null ) {
			try {
				resultado += this.getPrograma( ).getCanonicalPath( );
			} catch ( IOException e ) {
				resultado += this.getPrograma( ).getName( );
				e.printStackTrace( );
			}

			if ( this.getListaArgumentos( ) != null ) {
				Iterator < String > it = this.getListaArgumentos( ).iterator( );
				while ( it.hasNext( ) ) {
					resultado += " " + it.next( );
				}
			}
		} else {
			resultado = null;
		}

		return resultado;
	}

	public void setFicheroResultados ( File f ) {
		this.ficheroResultados = f;
		OnChange();		
	}
	
	public File getFicheroResultados( ) {
		return ficheroResultados;
	}

	public boolean isEjecuted ( ) {
		File fich = this.getFicheroResultados( );
		File prog = this.getPrograma( );
		return fich != null && fich.exists( ) && prog != null && prog.exists( );
	}

	public void addEntrada( String e ){
		if ( e != null && e.compareTo( "" ) != 0 ) {
			this.getEntradasListModel( ).addElement( e );
		}
		this.OnChange( );
	}
	
	public void addArgumento( String a ){
		if ( a != null && a.compareTo( "" ) != 0 ) {
			this.getArgumentosListModel( ).addElement( a );
		}
		this.OnChange( );
	}

	public void removeArgumento ( Object object ) {
		this.getArgumentosListModel( ).removeElement( object );
		
	}

	public void removeEntrada ( Object object ) {
		this.getEntradasListModel( ).removeElement( object );
		
	}
}
