/**
 * 
 */
package modelo.vestadisticas.grafollamadas;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import java.util.Map;
import java.util.ResourceBundle;
import java.util.Vector;


import javax.swing.JComboBox;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import negocio.PreferenciasSingleton;

import modelo.modeloXML.Ciclo;
import modelo.modeloXML.EntradaFuncion;
import modelo.modeloXML.InterfazModeloXML;
import modelo.modeloXML.ModeloXML;
import modelo.vestadisticas.ModeloEstadistica;


/**
 * @author angel
 *
 */
public class ModeloArbolGrafoLlamadas implements TreeModel, InterfazModeloXML, ActionListener, TreeSelectionListener {

	ModeloXML moXML;
	
	TreeNode root = null;
	
	private ResourceBundle propiedades = ResourceBundle.getBundle("interfaz.idiomas.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	
	private LinkedList <TreeModelListener> listeners = new LinkedList<TreeModelListener>();
	
	public ModeloArbolGrafoLlamadas (ModeloEstadistica x) {
		setDatosXML( x );
	}
	
	public ModeloArbolGrafoLlamadas () {
	}
	
	
	/**
	 * Crear el Árbol de Entradas.
	 * @param e entradaFuncion a partir de la que se va a continuar el árbol.
	 * @param entradasFuncion Map que contiene todas las entradasFuncion.
	 * @param entradasFuncionHechas Map que contiene todas las entradaFuncion ya procesadas.
	 * @return
	 */
	protected DefaultMutableTreeNode crearIndice ( EntradaFuncion e, 
			Map < Integer, EntradaFuncion > entradasFuncion, 
			Map < Integer, DefaultMutableTreeNode > entradasFuncionHechas,
			Map < EntradaFuncion, Boolean > ancestros) {
		DefaultMutableTreeNode Ae;
		
		
		/*recursión INICIO*/
		Ae = null;
		/*Primera comprobación:*/
		/*1- La entradaFuncion e ya ha generado su nodo.*/
		if( entradasFuncionHechas.containsKey(e.getIndice()) ) {
			if( e.getHijos().isEmpty() ) {
				/*caso base1, hecha sin hijos: Es simplemente ese nodo. Se copia. */
				Ae = (DefaultMutableTreeNode) entradasFuncionHechas.get( e.getIndice() ).clone();				
			}
			else {
				/*caso base2, hecha con hijos: Cuando es un árbol, el copiado no es tan trivial y hace falta una copia.*/
				DefaultMutableTreeNode copiandoAe = (DefaultMutableTreeNode) entradasFuncionHechas.get(e.getIndice());
				/*Hacer copia con todos los hijos*/
				Ae = copiarConHijos(copiandoAe);
			}

		}
		/*2- La entradaFuncion e no ha generado el nodo todavía.*/
		else {
			if ( e.getHijos().isEmpty() && e.getHijosMiembroCiclo( ).isEmpty( ) ) {
				/*caso base3, no hecha pero sin hijos: Es simplemente ese nodo. Se genera el nodo.*/
				Ae = new DefaultMutableTreeNode ( e );
			}
			else {
				/*caso recursivo multiple, no hecha y con hijos: Hay que generar el nodo de esta entrada y seguir la recursión.*/
				Ae = new DefaultMutableTreeNode ( e );
				Vector <EntradaFuncion> todasLasEntradas = new Vector <EntradaFuncion>();
				todasLasEntradas.addAll( e.getHijos( ) );
				todasLasEntradas.addAll( e.getHijosMiembroCiclo( ) );
				Iterator < EntradaFuncion > i = todasLasEntradas.iterator();
				while ( i.hasNext() ) {
					EntradaFuncion hijo = i.next();
					if ( ancestros.get( hijo ) == Boolean.FALSE ) {
						Map <EntradaFuncion, Boolean > ancestrosHijo = new HashMap < EntradaFuncion , Boolean > ( ancestros ); 
						ancestrosHijo.put( e , Boolean.TRUE );
						Ae.add( crearIndice( hijo, entradasFuncion, entradasFuncionHechas, ancestrosHijo ) );
					}
					/* Caso base 4, este hijo ya ha salido como ancestro: Simplemente se crea el nodo como si no tuviera descendencia. */
					else {
						Ae.add( new DefaultMutableTreeNode ( hijo ) );
					}
				}
			}
			/*Cuando se termina de generar, se pone el nodo como creado. Termina el caso recursivo, listos todos los hijos*/
			/*Cuando la entradaFuncion e, forma parte de un ciclo, no puede estar hecha, puesto que cambia según los ancestros.*/
			if( !e.hasCiclo() ) {
				entradasFuncionHechas.put(e.getIndice(), Ae);
			}
			
		}
		/*recursión FIN*/
		return Ae;
	}
	
	/**
	 * @param copiandoAe
	 * @param ae
	 */
	private DefaultMutableTreeNode copiarConHijos( DefaultMutableTreeNode copiandoAe ) {
		DefaultMutableTreeNode ae;
		//copiamos el nodo actual
		ae = (DefaultMutableTreeNode) copiandoAe.clone();
		if ( !copiandoAe.isLeaf() ) {
			//si no es hoja, copiamos todos sus hijos recursivamente
			int numeroDeHijos = copiandoAe.getChildCount();
			for ( int i = 0 ; i < numeroDeHijos ; i++ ) {
				DefaultMutableTreeNode copiandoHijo = (DefaultMutableTreeNode) copiandoAe.getChildAt(i);
				DefaultMutableTreeNode hijo = null;
				//recursión al hijo
				hijo = copiarConHijos(copiandoHijo);
				ae.add(hijo);
			}
			
		}
		//fin de la copia de este nodo y todos sus hijos si hubiere
		return ae;
		
	}
	

	/*****************************************************************************/
	
	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeModel#addTreeModelListener(javax.swing.event.TreeModelListener)
	 */
	public void addTreeModelListener(TreeModelListener l) {
		listeners.add(l);
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeModel#getChild(java.lang.Object, int)
	 */
	public Object getChild(Object parent, int index) {
		TreeNode hijo = null;
		if (parent instanceof TreeNode) {
			hijo = ((TreeNode)parent).getChildAt(index);
		}
		return hijo;
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeModel#getChildCount(java.lang.Object)
	 */
	public int getChildCount(Object parent) {
		int hijos = 0;
		if (parent instanceof TreeNode) {
			hijos = ((TreeNode) parent).getChildCount();
		}
		return hijos;
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeModel#getIndexOfChild(java.lang.Object, java.lang.Object)
	 */
	public int getIndexOfChild(Object parent, Object child) {
		int indice;
		if (parent == null || child == null)
			indice = -1;
		else {
			if (parent instanceof TreeNode && child instanceof TreeNode) {
				indice = ((TreeNode) parent).getIndex((TreeNode) child);
			} else
				indice = -1;
		}
		
		return indice;
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeModel#getRoot()
	 */
	public Object getRoot() {
		
		return root;
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeModel#isLeaf(java.lang.Object)
	 */
	public boolean isLeaf(Object node) {
		boolean b = false;
		if( node instanceof TreeNode) {
			b = ((TreeNode) node).isLeaf();
		}
		return b;
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeModel#removeTreeModelListener(javax.swing.event.TreeModelListener)
	 */
	public void removeTreeModelListener(TreeModelListener l) {
		listeners.remove(l);
	}

	/* (non-Javadoc)
	 * @see javax.swing.tree.TreeModel#valueForPathChanged(javax.swing.tree.TreePath, java.lang.Object)
	 */
	public void valueForPathChanged(TreePath path, Object newValue) {
		
	}
	
	/***************************************************************************/
	/* (non-Javadoc)
	 * @see componentes.ModeloXML#setXML(org.w3c.dom.Document)
	 */
	public void setDatosXML( ModeloEstadistica me ) {
		ModeloXML x = me.getModeloXML( );
		if ( x != null && x.hasGrafoLlamadas() ) {
			moXML = x;
			
			DefaultMutableTreeNode raiz = new DefaultMutableTreeNode( propiedades.getString( "eti_MoArb_raiz") );
			
			this.root = raiz;
			
			Map < Integer, EntradaFuncion > entradasFuncion = moXML.getEntradasFuncion();
			
			HashMap < Integer, DefaultMutableTreeNode > entradasFuncionHechas = 
				new HashMap < Integer, DefaultMutableTreeNode > ( entradasFuncion.size() );
			
			HashMap < EntradaFuncion, Boolean > ancestros = new HashMap < EntradaFuncion, Boolean > (entradasFuncion.size() );  
			
			Iterator < EntradaFuncion > j = entradasFuncion.values().iterator();
			while ( j.hasNext( ) ) {
				EntradaFuncion funcion = j.next( );
				ancestros.put( funcion , Boolean.FALSE );
			}
			j = entradasFuncion.values().iterator();
			while( j.hasNext() ) {
				EntradaFuncion funcion = j.next();
				if ( funcion.isEspontanea( ) ) {
					raiz.add ( crearIndice( funcion, entradasFuncion , entradasFuncionHechas, new HashMap < EntradaFuncion, Boolean > ( ancestros ) ) );
				}
			}
			
			
			fireTreeStructureChange();
		}
	}
	
	/**
	 * 
	 */
	private void fireTreeStructureChange ( ) {
		if ( this.getRoot( ) != null ) {
			TreeModelEvent evento = new TreeModelEvent( this , new TreePath(
					this.getRoot( ) ) );

			int i;

			for ( i = 0 ; i < listeners.size( ) ; i++ )
				( (TreeModelListener) listeners.get( i ) )
						.treeStructureChanged( evento );
		}
	}
	
	@Override
	public void actionPerformed ( ActionEvent e ) {
/*		EntradaFuncion fichero = null;

		if ( e.getSource( ) instanceof JComboBox ) {
			JComboBox x = (JComboBox ) e.getSource( );
			if( x.getSelectedItem( ) instanceof EntradaFuncion ) {
				fichero = (EntradaFuncion) ( x.getSelectedItem( ) );
				if ( fichero != null ) {
					setFuncionActivo( fichero );
				}
			}
		}*/
		EntradaFuncion fichero = null;
		Ciclo ciclo = null;

		if ( e.getSource( ) instanceof JComboBox ) {
			JComboBox x = (JComboBox ) e.getSource( );
			Object o = x.getSelectedItem( );
			if( o instanceof EntradaFuncion ) {
				fichero = (EntradaFuncion) o;
				if ( fichero != null ) {
					setFuncionActivo( fichero );
				}
			} else if ( o instanceof Ciclo ) {
				ciclo = ( Ciclo ) o;
				if ( ciclo != null ) {
					setCicloActivo( ciclo );
				}
			}
		}
	}
	
	
/** Sirve para que cuando se seleccione un nodo, se "active" esa funcion y que pueda salir resaltada en todos los sitios donde aparezca. */		
	@Override
	public void valueChanged(TreeSelectionEvent e) {
		
		// Obtenemos el path seleccionado
		TreePath path = e.getPath();
		Object nodo = path.getLastPathComponent( );
		if ( nodo instanceof EntradaFuncion ) {
			setFuncionActivo( (EntradaFuncion) nodo );
		}
	}
	
	/** FuncionActivo */
	private EntradaFuncion funcionActivo = null;
	private Ciclo cicloActivo = null;
	
	public void setFuncionActivo( EntradaFuncion e ) {
		/*EntradaFuncion funcionActivoPrevia = funcionActivo;*/
		funcionActivo = e;
		cicloActivo = null;
		/*if ( !(funcionActivoPrevia != null && e != null && funcionActivoPrevia.getIndice( ) == e.getIndice( ) ) ) {
			//fire lo que sea
			fireCambios( funcionActivo );
		}
		if( funcionActivo != null )*/
		fireCambios( /*funcionActivo*/ );
	}
	
	public void setCicloActivo ( Ciclo c ) {
		funcionActivo = null;
		cicloActivo = c;
		
		fireCambios(/* cicloActivo */);
	}
	
	public EntradaFuncion getFuncionActivo ( ) {
		return funcionActivo;
	}
	
	public Ciclo getCicloActivo ( ) {
		return cicloActivo;
	}
	
	private void fireCambios ( /*EntradaFuncion e*/ ) {
		if ( this.getRoot( ) != null ) {
			TreeModelEvent evento = new TreeModelEvent( this , new TreePath(
					this.getRoot( ) ) );

			int i;

			for ( i = 0 ; i < listeners.size( ) ; i++ )
				( (TreeModelListener) listeners.get( i ) )
						.treeNodesChanged( evento );
		}
	}

}
