/**
 * 
 */
package modelo.vestadisticas.grafollamadas;

import modelo.modeloXML.*;
import modelo.vestadisticas.ModeloEstadistica;
import negocio.PreferenciasSingleton;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import java.util.ResourceBundle;

import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.TableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;


/**
 * @author angel
 *
 */
public class ModeloTablaFuncionSenalada implements TableModel, InterfazModeloXML, ListSelectionListener, ActionListener, TreeSelectionListener {
	private final static long serialVersionUID = 1;

	private ModeloXML moXML;
	
	private ArrayList<TableModelListener> arrayListeners = new ArrayList<TableModelListener>();
	
	private ResourceBundle propiedades = ResourceBundle.getBundle("interfaz.idiomas.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );

	private final String[] nombreDeColumnas = { 
			propiedades.getString("eti_MoGL_nombre"),
			propiedades.getString("eti_MoGL_porcentaje"),
			propiedades.getString("eti_MoGL_tpropio"),
			propiedades.getString("eti_MoGL_thijos"),
			propiedades.getString("eti_MoGL_llamadas"),
			propiedades.getString("eti_MoGL_llamadasR"),
			propiedades.getString("eti_MoGL_indice"),
			propiedades.getString( "eti_MoGL_ciclo" )};
	
	public final static int NOMBRE = 0;
	public final static int PORCENTAJE = 1;
	public final static int TPROPIO = 2;
	public final static int THIJOS = 3;
	public final static int LLAMADAS = 4;
	public final static int LLAMADASRECURSIVAS =5;
	public final static int INDICE = 6;
	public final static int CICLO = 7;
	
	
	private double tiempoTotal = 0;
	
	//private Integer indice = null;
	private EntradaFuncion funcion = null;
	private Ciclo ciclo = null;


	public ModeloTablaFuncionSenalada ()  {
	}

	/**
	 * Crea el modelo de la tabla a partir del modelo de datos ModeloXML.
	 * @param x ModeloXML que contiene toda la información.
	 */
	public ModeloTablaFuncionSenalada (ModeloEstadistica x) {
		setDatosXML( x );
	}

	
	/**************METODOS**************/
	public void addTreeModelListener(TableModelListener l) {
		arrayListeners.add(l);
	}

	public void removeTreeModelListener(TableModelListener l) {
		arrayListeners.remove(l);
	}

	
	public int getColumnCount() {
		return nombreDeColumnas.length;
	}

	public int getRowCount() {
		int result = 0;
		if ( moXML != null && (funcion != null || ciclo != null ) ) {
			result = 1;
		}
		return result;
	}

	public String getColumnName(int col) {
		return nombreDeColumnas[col];
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnClass(int)
	 */
	public Class<?> getColumnClass( int columnIndex ) {
		Class <?> result = String.class;
		switch (columnIndex) {
		case NOMBRE: result = String.class; break;
		case PORCENTAJE: result = Double.class; break;
		case TPROPIO: result = Double.class; break;
		case THIJOS: result = Double.class; break;
		case LLAMADAS: result = Integer.class; break;
		case LLAMADASRECURSIVAS: result = Integer.class; break;
		case INDICE: result = Integer.class; break;
		case CICLO: result = Integer.class; break;
		}

		return result;
	}
	
	public boolean isFuncionSelected ( ) {
		boolean resultado = false;
		
		if ( funcion != null && ciclo == null )
			resultado = true;
		
		return resultado;
	}
	
	public boolean isCicloSelected ( ) {
		boolean resultado = false;
		
		if ( funcion == null && ciclo != null ) {
			resultado = true;
		}
		
		return resultado;
	}

	
	
	public Object getValueAt(int row, int col) {
		Object result = null;
		if ( moXML != null && row >= 0 && row < getRowCount() && col >= 0 && col < getColumnCount() ) {
			if ( isFuncionSelected( ) )
				switch (col) {
				case NOMBRE: result = funcion.getNombreFuncion(); break;
				case PORCENTAJE: result = ( this.tiempoTotal != 0 ) ?  (funcion.getTiempoHijos( ) + funcion.getTiempoPropio( ) ) / this.tiempoTotal: new Integer(0); break;
				case TPROPIO: result = funcion.getTiempoPropio(); break;
				case THIJOS: result = funcion.getTiempoHijos(); break;
				case LLAMADAS: result = funcion.getNumeroLlamadas(); break;
				case LLAMADASRECURSIVAS: result = funcion.getNumeroLlamadasRecursivas(); break;
				case INDICE: result = funcion.getIndice(); break;
				case CICLO: result = funcion.hasCiclo( )? funcion.getCiclo( ).getCiclo( ) : null; break;
				}
			else if ( isCicloSelected( ) ) {

				switch (col) {
				case NOMBRE: result = propiedades.getString("flr_ciclo") + " < " + ciclo.getCiclo() + " >"; break;
				case PORCENTAJE: result = ( this.tiempoTotal != 0 ) ?  (ciclo.getTiempoHijos( ) + ciclo.getTiempoPropio( ) ) / this.tiempoTotal: new Integer(0); break;
				case TPROPIO: result = ciclo.getTiempoPropio(); break;
				case THIJOS: result = ciclo.getTiempoHijos(); break;
				case LLAMADAS: result = ciclo.getNumeroLlamadas(); break;
				case LLAMADASRECURSIVAS: result = ciclo.getNumeroLlamadasRecursivas(); break;
				case INDICE: result = ciclo.getIndice(); break;
				case CICLO: result = ciclo.getCiclo( ); break;
				}
			}
		}
		return result;
		//TODO porcentaje
	}


	public boolean isCellEditable(int row, int col) {
		return false;
	}

	public void setValueAt(Object value, int row, int col) {
		//Insertar. Esta tabla es de solo lectura. No necesita hacer nada.
	}
	
	/**
	 * 
	 */
	private void fireTableDataChanged( ) {
		TableModelEvent evento = new TableModelEvent( this );
		
		int i;

		for ( i=0; i<arrayListeners.size( ); i++ )
		    ( ( TableModelListener ) arrayListeners.get( i ) ).tableChanged( evento );		
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#addTableModelListener(javax.swing.event.TableModelListener)
	 */
	public void addTableModelListener(TableModelListener l) {
		
		arrayListeners.add(l);
	}


	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#removeTableModelListener(javax.swing.event.TableModelListener)
	 */
	public void removeTableModelListener(TableModelListener l) {
		arrayListeners.remove(l);		
	}

	public void setDatosXML( ModeloEstadistica me ) {
		ModeloXML x = me.getModeloXML( );
		if ( x != null && x.hasGrafoLlamadas() ) {
			moXML = x;
			fireTableDataChanged();
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
	 */
	public void valueChanged(ListSelectionEvent e) {
		EntradaFuncion fichero = null;
		Ciclo ciclo = null;
		
		if(e.getSource() instanceof JList ) {
			Object o = ((JList)e.getSource()).getSelectedValue();	

			if( o instanceof EntradaFuncion ) {
				fichero = (EntradaFuncion) o;
				if ( fichero != null ) {
					setFuncionActivo( fichero.getIndice( ) );
				} 
			}else if ( o instanceof Ciclo ) {
				ciclo = (Ciclo) o;
				if ( ciclo != null ) {
					setCicloActivo ( ciclo.getIndice( ) );
				}

			}
		}
	}
	
	/**
	 * @param nombreFuncion
	 */
	public void setFuncionActivo( Integer indiceFuncion ) {
		if( moXML != null && moXML.hasGrafoLlamadas() && moXML.getEntradasFuncion().containsKey(indiceFuncion) ) {
			this.funcion = moXML.getEntradasFuncion().get(indiceFuncion);
			this.ciclo = null;
			this.tiempoTotal = moXML.getTiempoEjecucion( ) != null ? moXML.getTiempoEjecucion( ).doubleValue( ): 0; 
			this.fireTableDataChanged();
		}
		//TODO tiempo Total¿?¿?¿?¿?¿?¿?¿?¿
	}
	
	public void setCicloActivo( Integer indiceCiclo ) { 
		if( moXML != null && moXML.hasGrafoLlamadas() && moXML.getCiclos( ).containsKey(indiceCiclo) ) {
			this.ciclo = moXML.getCiclos().get( indiceCiclo );
			this.funcion = null;
			this.tiempoTotal = moXML.getTiempoEjecucion( ) != null ? moXML.getTiempoEjecucion( ).doubleValue( ): 0; 
			this.fireTableDataChanged();

		}
	}
	

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
	 */
	public void valueChanged(TreeSelectionEvent e) {
		EntradaFuncion funcion = null;
		Ciclo ciclo = null;
		
		TreePath tp = e.getPath();

		DefaultMutableTreeNode n = null;
		if(tp.getLastPathComponent() instanceof DefaultMutableTreeNode) {
			n = (DefaultMutableTreeNode) tp.getLastPathComponent();
			Object o = n.getUserObject() ; 
			if( o instanceof EntradaFuncion ) {
				funcion = (EntradaFuncion) o;
				if ( funcion != null ) {
					setFuncionActivo( funcion.getIndice( ) );
				} 
			}else if ( o instanceof Ciclo ) {
				ciclo = (Ciclo) o;
				if ( ciclo != null ) {
					setCicloActivo ( ciclo.getIndice( ) );
				}
				
			}
		}
	}

	@Override
	public void actionPerformed ( ActionEvent e ) {
		EntradaFuncion funcion = null;
		Ciclo ciclo = null;

		if ( e.getSource( ) instanceof JComboBox ) {
			JComboBox x = (JComboBox ) e.getSource( );
			Object o = x.getSelectedItem( );
			if( o instanceof EntradaFuncion ) {
				funcion = (EntradaFuncion) o;
				if ( funcion != null ) {
					setFuncionActivo( funcion.getIndice( ) );
				} 
			} else if ( o instanceof Ciclo ) {
				ciclo = (Ciclo) o;
				if ( ciclo != null ) {
					setCicloActivo ( ciclo.getIndice( ) );
				}
				
			}
		}
	}
}

