/**
 * 
 */
package modelo.vestadisticas.grafollamadas;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.TableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import negocio.PreferenciasSingleton;

import modelo.modeloXML.Ciclo;
import modelo.modeloXML.EntradaFuncion;
import modelo.modeloXML.InterfazModeloXML;
import modelo.modeloXML.ModeloXML;
import modelo.vestadisticas.ModeloEstadistica;


/**
 * @author angel
 *fichero
 */
public class ModeloTablaFuncionesLlamadas implements TableModel, InterfazModeloXML, ListSelectionListener, ActionListener, TreeSelectionListener {
	private final static long serialVersionUID = 1;

	private ModeloXML moXML;
	
	private ArrayList<TableModelListener> arrayListeners = new ArrayList<TableModelListener>();
	
	private ResourceBundle propiedades = ResourceBundle.getBundle("interfaz.idiomas.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );

	private final String[] nombreDeColumnas = { 
			propiedades.getString("eti_MoGL_nombre"),
			/*propiedades.getString("eti_MoGL_porcentaje"),*/
			propiedades.getString("eti_MoGL_tpropioH"),
			propiedades.getString("eti_MoGL_thijosH"),
			propiedades.getString("eti_MoGL_llamadas"),
			propiedades.getString("eti_MoGL_indice"),
			propiedades.getString( "eti_MoGL_ciclo" )};
	
	public final static int NOMBRE = 0;
	/*public final static int PORCENTAJE = 1;
	public final static int TPROPIO = 2;
	public final static int THIJOS = 3;
	public final static int LLAMADAS = 4;
	public final static int INDICE = 5;*/
	public final static int TPROPIO = 1;
	public final static int THIJOS = 2;
	public final static int LLAMADAS = 3;
	public final static int INDICE = 4;
	public final static int CICLO = 5;
	
	//private Integer indice = null;
	private EntradaFuncion funcion = null;
	private Ciclo ciclo = null;
	private Vector < EntradaFuncion > funcionesLlamadas = new Vector < EntradaFuncion > (0);
	private Vector < EntradaFuncion > funcionesLlamadasMiembro = new Vector < EntradaFuncion > (0);
	
	public ModeloTablaFuncionesLlamadas () {

	}

	/**
	 * Crea el modelo de la tabla a partir del modelo de datos ModeloXML.
	 * @param x ModeloXML que contiene toda la información.
	 */
	public ModeloTablaFuncionesLlamadas (ModeloEstadistica x) {
		setDatosXML(x);
	}

	/**************METODOS**************/
	public void addTreeModelListener(TableModelListener l) {
		arrayListeners.add(l);
	}

	public void removeTreeModelListener(TableModelListener l) {
		arrayListeners.remove(l);
	}

	
	public int getColumnCount() {
		return nombreDeColumnas.length;
	}

	public int getRowCount() {
		int result = 0;
		if ( moXML != null && ( ciclo != null || funcion != null ) ) {
			result = funcionesLlamadasMiembro.size();  
			result += funcionesLlamadas.size(); 
		}
		return result;	}

	public String getColumnName(int col) {
		return nombreDeColumnas[col];
	}

	public boolean isFuncionSelected ( ) {
		boolean resultado = false;
		
		if ( funcion != null && ciclo == null )
			resultado = true;
		
		return resultado;
	}
	
	public boolean isCicloSelected ( ) {
		boolean resultado = false;
		
		if ( funcion == null && ciclo != null ) {
			resultado = true;
		}
		
		return resultado;
	}
	
	
	/* Devuelve la entradaFuncion que corresponde a la row.
	 * Las primeras filas serán las funciones miembro.
	 * El resto serán las funciones no miembros. */
	public EntradaFuncion getEntradaFuncionRow( int row ) {
		EntradaFuncion resultado = null;
		
		if ( row >= 0 && row < getRowCount() ) {
			if ( row < funcionesLlamadasMiembro.size( ) )
				resultado = funcionesLlamadasMiembro.get( row );
			else
				resultado = funcionesLlamadas.get( row - funcionesLlamadasMiembro.size( ) );
		}
		
		return resultado;
	}
	
	
	/* Devuelve true si row es menor que el número de funciones miembro.
	 * Así las primeras filas serán las funciones miembro. */
	public boolean isRowFuncionMiembro( int row ) {
		boolean resultado = false;
		
		int miembros = funcionesLlamadasMiembro.size();
		if ( row < miembros || row >= 0 ) {
			resultado = true;
		}
		
		return resultado;
	}
	
	/* Devuelve true si row está entre el número de funciones miembro y
	 * el número de funciones totales.
	 * Así las primeras filas serán las funciones miembro. */
	public boolean isRowFuncionNoMiembro( int row ) {
		boolean resultado = false;
		
		int miembros = funcionesLlamadasMiembro.size();
		if ( row >= miembros || row < getRowCount() ) {
			resultado = true;
		}
		
		return resultado;
	}
	
	public Object getValueAt(int row, int col) {
		Object result = null;
		if ( moXML != null && row >= 0 && row < getRowCount() && col >= 0 && col < getColumnCount() ) {
			EntradaFuncion f = getEntradaFuncionRow( row );
			if ( isFuncionSelected( ) ) {
				switch (col) {
				case NOMBRE: result = f.getNombreFuncion(); break;
				/*case PORCENTAJE: 
					//double t = funcion.getTiempoHijos( ); result = (row >= miembros ) ? ( t != 0 ? ( funcion.getEstimacionTiempoHijosHijo( funcionesLlamadas.get(row-miembros) ) + funcion.getEstimacionTiempoPropioHijo( funcionesLlamadas.get(row-miembros) ) ) / t : new Double(0) ) : null; break;
					result = funcion.getPorcentajeLlamadas( f ); break;*/
				case TPROPIO: result = isRowFuncionNoMiembro( row ) ? funcion.getEstimacionTiempoPropioHijo( f ): null; break;
				case THIJOS: result = isRowFuncionNoMiembro( row ) ? funcion.getEstimacionTiempoHijosHijo( f ): null; break;
				case LLAMADAS: result = funcion.getNumeroLlamadasHijo( f ); break;
				case INDICE: result = f.getIndice(); break;
				case CICLO: result = f.hasCiclo( )? f.getCiclo( ).getCiclo( ) : null; break;
				}
			} else if ( isCicloSelected( ) ) {
				switch (col) {
				case NOMBRE: result = f.getNombreFuncion(); break;
				/*case PORCENTAJE: 
					double t = ciclo.getTiempoHijos( ); 
					result = (row >= miembros ) ? 
							( t != 0 ? 
									( ciclo.getEstimacionTiempoHijosHijo( 
											funcionesLlamadas.get(row-miembros) ) 
											+ funcion.getEstimacionTiempoPropioHijo( 
												funcionesLlamadas.get(row-miembros) ) ) 
												/ t 
											: new Double(0) ) 
											: null;
					result = ciclo.getPorcentajeLlamadas( f );
									break;*/
				case TPROPIO: 
					result = isRowFuncionNoMiembro( row ) ? 
						ciclo.getEstimacionTiempoPropioHijo( f ): 
									null; 
					break;
				case THIJOS: 
					result = isRowFuncionNoMiembro( row ) ? ciclo.getEstimacionTiempoHijosHijo( f ): null; break;
				case LLAMADAS: 
					result = ciclo.getNumeroLlamadasHijo( f ); 
					break;
				case INDICE: 
					result = f.getIndice(); 
					break;
				case CICLO: result = f.hasCiclo( )? f.getCiclo( ).getCiclo( ) : null; break;
				}
			}
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnClass(int)
	 */

	public Class <?> getColumnClass( int columnIndex ) {
		Class <?> result = String.class;
		switch (columnIndex) {
		case NOMBRE: result = String.class; break;
		/*case PORCENTAJE: result = Double.class; break;*/
		case TPROPIO: result = Double.class; break;
		case THIJOS: result = Double.class; break;
		case LLAMADAS: result = Integer.class; break;
		case INDICE: result = Integer.class; break;
		case CICLO: result = Integer.class; break;
		}

		return result;
	}

	public boolean isCellEditable(int row, int col) {
		return false;
	}

	public void setValueAt(Object value, int row, int col) {
		/** Insertar. Esta tabla es de solo lectura. No necesita hacer nada.*/
	}
	
	/**
	 * 
	 */
	private void fireTableDataChanged() {
		
		TableModelEvent evento = new TableModelEvent(this);
		
		int i;

		for (i=0; i<arrayListeners.size(); i++)
		    ((TableModelListener)arrayListeners.get(i)).tableChanged(evento);
		
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#addTableModelListener(javax.swing.event.TableModelListener)
	 */
	public void addTableModelListener(TableModelListener l) {
		
		arrayListeners.add(l);
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#removeTableModelListener(javax.swing.event.TableModelListener)
	 */
	public void removeTableModelListener(TableModelListener l) {
		arrayListeners.remove(l);		
	}

	public void setDatosXML(ModeloEstadistica me ) {
		ModeloXML x = me.getModeloXML( );
		if ( x != null && x.hasGrafoLlamadas() ) {
			moXML = x;
			fireTableDataChanged();
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
	 */
	public void valueChanged(ListSelectionEvent e) {
		EntradaFuncion fichero = null;
		Ciclo ciclo = null;
		

		if(e.getSource() instanceof JList ) {
			Object o = ((JList)e.getSource()).getSelectedValue();	

			if( o instanceof EntradaFuncion ) {
				fichero = (EntradaFuncion) o;
				if ( fichero != null ) {
					setFuncionActivo( fichero.getIndice( ) );
				} 
			}else if ( o instanceof Ciclo ) {
				ciclo = (Ciclo) o;
				if ( ciclo != null ) {
					setCicloActivo ( ciclo.getIndice( ) );
				}

			}
		}
	}

	/**
	 * @param string
	 */
	private void setFuncionActivo( Integer indiceFuncion ) {
		
		if( moXML != null && moXML.hasGrafoLlamadas() && moXML.getEntradasFuncion().containsKey(indiceFuncion) ) {
			this.funcion = moXML.getEntradasFuncion().get(indiceFuncion);
			this.ciclo = null;
			this.funcionesLlamadas = new Vector < EntradaFuncion > ( funcion.getHijos() );
			this.funcionesLlamadasMiembro = new Vector < EntradaFuncion > ( funcion.getHijosMiembroCiclo());
			fireTableDataChanged();
		}		
	}
	
	public void setCicloActivo( Integer indiceCiclo ) { 
		if( moXML != null && moXML.hasGrafoLlamadas() && moXML.getCiclos( ).containsKey(indiceCiclo) ) {
			this.ciclo = moXML.getCiclos().get( indiceCiclo );
			this.funcion = null; 
			this.funcionesLlamadas = new Vector < EntradaFuncion > ( ciclo.getHijos() );
			this.funcionesLlamadasMiembro = new Vector < EntradaFuncion > ( ciclo.getMiembros() );
			// ??? cual de las dos :S
			this.fireTableDataChanged();
		}
	}
	
	/*public HashMap valores() {
		return datosEntradas;
	}*/

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
	 */
	public void valueChanged(TreeSelectionEvent e) {
		EntradaFuncion funcion = null;
		Ciclo ciclo = null;
		TreePath tp = e.getPath();


		DefaultMutableTreeNode n = null;
		if(tp.getLastPathComponent() instanceof DefaultMutableTreeNode) {
			n = (DefaultMutableTreeNode) tp.getLastPathComponent();
			Object o = n.getUserObject() ; 
			if( o instanceof EntradaFuncion ) {
				funcion = (EntradaFuncion) o;
				if ( funcion != null ) {
					setFuncionActivo( funcion.getIndice( ) );
				} 
			}else if ( o instanceof Ciclo ) {
				ciclo = (Ciclo) o;
				if ( ciclo != null ) {
					setCicloActivo ( ciclo.getIndice( ) );
				}
				
			}
		}
	}
	
	@Override
	public void actionPerformed ( ActionEvent e ) {

		EntradaFuncion funcion = null;
		Ciclo ciclo = null;

		if ( e.getSource( ) instanceof JComboBox ) {
			JComboBox x = (JComboBox ) e.getSource( );
			Object o = x.getSelectedItem( );
			if( o instanceof EntradaFuncion ) {
				funcion = (EntradaFuncion) o;
				if ( funcion != null ) {
					setFuncionActivo( funcion.getIndice( ) );
				} 
			}else if ( o instanceof Ciclo ) {
				ciclo = (Ciclo) o;
				if ( ciclo != null ) {
					setCicloActivo ( ciclo.getIndice( ) );
				}
				
			}
		}
	}

}

