/*
 * Clase guigprof.java es una aplicación Java 1.4.
 * Pertenece a la aplicación guigprof para el proyecto fin de carrera de 
 * Ángel García Fernández.
 * Tutor: Francisco Palomo Lozano e Inmaculada Medina Bulo
 * Fecha: Agosto de 2007.
 */

package negocio;

import interfaz.GUI;
import interfaz.dialogos.error.DialogoError;

import java.awt.Container;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Locale;
import java.util.ResourceBundle; //L&F

import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.help.HelpSetException;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import datos.SesionXMLParser;

import modelo.Sesion;

/**
 * Es la clase que lanza la aplicación (main).
 * 
 * @author Ángel García Fernández
 * @version 0.1
 * @since 0.0.1
 */
public class guigprof extends Object {

	final static long	serialVersionUID	= 5L;

	/** 
	 * ventana principal del programa
	 */
	private static GUI ventanaPrincipal;

	///** Preferencias de la aplicación */
	//private static Preferencias p = new Preferencias();
	
	/** Directorio de usuario */
	private static File userDir = new File ( System.getProperty( "user.home" ) ); //"user.home" "user.dir"
	/** Directorio del programa ~/.guigprof */
	private static File directorioGUIgprof = new File ( userDir, ".guigprof" );
	/** Fichero de configuración ~/.guigprof/preferenciasGUIgprof */
	private static File ficheroConfiguracion = new File ( directorioGUIgprof, "preferenciasGUIgprof.xml");
	/** Resguardo del fichero de configuración ~/.guigprof/preferenciasGUIgprof.backup */
	private static File ficheroConfiguracionBackUp = new File( directorioGUIgprof , "preferenciasGUIgprof.backup.xml" );
	
	/** recursos para dar soporte de idiomas sobre errores */
	static ResourceBundle		errores				= ResourceBundle
													.getBundle( "interfaz.errores.Errores", Locale.getDefault() );

	/** recursos para dar soporte de idiomas sobre errores */
	static ResourceBundle		idiomas				= ResourceBundle
													.getBundle( "interfaz.idiomas.Recursos", Locale.getDefault() );


	/** Título de la ventana de error */
	static String				TITULOVENTANAERROR	= "guigprof - " + errores.getString("wordError");
	
	/** Esta propiedad sirve para implementar el singleton. */
    private static guigprof INSTANCE = null;
    
	/** guigprof es un singleton */
    public static guigprof getInstance() {
        if (INSTANCE == null) INSTANCE = new guigprof( );
        return INSTANCE;
    }
	
    /** Un singleton no debe clonarse*/	
    public Object clone() throws CloneNotSupportedException {
        throw new CloneNotSupportedException(); 
	}
    
	/**
	 * Crear la clase guigprof que contendrá todos los métodos de este paquete a
	 * modo de fachada. Carga las preferencias. Crea la ventana principal.
	 * Maneja posibles errores.
	 */
	private guigprof ( ) {

		/** Cargar fichero de preferencias de usuario.*/
		try {
			if ( ficheroConfiguracion.exists( ) ) {
				PreferenciasSingleton.getPreferenciasSingleton( ).cargarPreferencias( ficheroConfiguracion );
			}
		} catch ( Exception e ) {
			e.printStackTrace( );
			DialogoError.getInstance( ).showErrorDialog( 
					errores.getString( "err_errorLecturaXMLPreferencias" ) );
		}
		
		/** obtener los resources a través del Locale de las preferencias. */
		
		errores	= ResourceBundle.getBundle( "interfaz.errores.Errores", 
				PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );

		idiomas	= ResourceBundle.getBundle( "interfaz.idiomas.Recursos", 
				PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
		
		/** además añadir como idioma Locale para todos los componentes el mismo que el 
		 * de la aplicación. */
		JComponent.setDefaultLocale(
				PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
		
		/** crear y mostrar la GUI de esta aplicación.
		 * Todo lo que conlleve interactuar con el GUI debe hacerse
		 * desde el EDT.
		 * El main tiene su propio hilo que no es el EDT. */
		SwingUtilities.invokeLater( new Runnable( ) {

			public void run ( ) {
				/** Coger el Look And Feel guardado como preferencia si existe alguno. */
				try {
					JDialog.setDefaultLookAndFeelDecorated( true );
					JFrame.setDefaultLookAndFeelDecorated( true );
					
					/** Comprobar que el look n feel existe y usarlo */
					String laf = PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLookAndFeelName( );
					UIManager.LookAndFeelInfo [] lyf_info = UIManager.getInstalledLookAndFeels();
					int sel = -1;
					
					for ( int i = 0 ; i < lyf_info.length ; i ++ ) {
						String lyf_instalado = lyf_info[i].getName( );
						if( lyf_instalado.equals( laf )) {
							sel = i;
							laf = lyf_info[i].getClassName( );
							break;
						}
					}
					if ( sel < 0 ) {
						String opt1 = idiomas.getString( "lyf_crosplatform" );
						String opt2 = idiomas.getString( "lyf_system" );
						Object [] opcion_lyf = { opt1, opt2 };
						
						String res = (String) JOptionPane.showInputDialog( 
								null , 
								idiomas.getString( "eti_NoL&F" ) , getGUI().getTitle( ) , 
								JOptionPane.INFORMATION_MESSAGE , null , opcion_lyf , opt1 );
						
						if ( res.equals( opt1 ) )
							laf = UIManager.getCrossPlatformLookAndFeelClassName( );
						else 
							laf = UIManager.getSystemLookAndFeelClassName( );
					}
					
					UIManager.setLookAndFeel( laf );
					
				} catch ( Exception e ) {
					String error = errores.getString( "errl&f" );
					DialogoError.getInstance( ).showErrorDialog( error );
					System.err.println( error );
					e.printStackTrace( );
				}
				
				/** Arrancar el GUI */
				try {
					ventanaPrincipal = new GUI(  );
				} catch ( Exception e ) {
					DialogoError.getInstance( ).showErrorDialog(  errores.getString( "err_ventanaPrincipal" ) );
	
					e.printStackTrace();
				}
			}
		} );		
	}

	public void cerrarAplicacion ( ) {
		
		/** crear el directorio del programa si no existe */
		if ( !directorioGUIgprof.exists( ) )
			if ( directorioGUIgprof.mkdir( ) )
				System.out.println( errores.getString( "programDirCreated" ) );
			else 
				System.out.println( errores.getString( "programDirNotCreated"));

		/** si existía o se creó con exito, crear el fichero de preferencias */
		if ( directorioGUIgprof.exists( ) ) {
		
			/** Si no existe se crea */
			if ( !ficheroConfiguracion.exists( ) )
			try {
				if ( ficheroConfiguracion.createNewFile( ) ) 
					System.out.println( errores.getString( "SetupFileCreated") );
				else
					System.out.println( errores.getString( "SetupFileNotCreated") );
			} catch ( IOException ex ) {
				System.out.println( errores.getString( "SetupFileNotCreated") );
				ex.printStackTrace();
			} 
			else { /** Si existía y han habido cambios en las preferencias, se guarda como backup y se crea el nuevo */
				if ( PreferenciasSingleton.getPreferenciasSingleton( ).isBeenModified( ) ) {
					if ( ficheroConfiguracionBackUp.exists( ) )
						ficheroConfiguracionBackUp.delete( );
					ficheroConfiguracion.renameTo( ficheroConfiguracionBackUp );

					try {
						if ( ficheroConfiguracion.createNewFile( ) )
							System.out.println( errores.getString( "SetupFileCreated") );
						else
							System.out.println( errores.getString( "SetupFileNotCreated") );
					} catch ( IOException ex ) {
						System.out.println( errores.getString( "SetupFileNotCreated") );
						ex.printStackTrace();
					}
				}
			}

			/** Nos aseguramos de que se terminó creando el nuevo con éxito y se
			 * guarda la configuración. */
			if ( ficheroConfiguracion.exists( ) ) 
			try {
				PreferenciasSingleton.getPreferenciasSingleton( ).guardarPreferencias( ficheroConfiguracion );
			} catch ( Exception e ){
				DialogoError.getInstance( ).showErrorDialog( 
						errores.getString( "err_errorLecturaXMLPreferencias" ) );
						e.printStackTrace();
						System.exit( 1 );
			}
			System.exit( 0 );
		}
	}
	
	
	/**
	 * Función principal, desde donde se lanza la aplicación. Se especifica el
	 * L&F
	 * 
	 * @param args
	 */
	public static void main ( String [] args ) {
		try {
			guigprof.getInstance( );
		} catch ( Exception e ) {
			DialogoError.getInstance( ).showErrorDialog( errores.getString( "err_inesperado" ) ); 
			e.printStackTrace( );
		}
	}
	
	public  GUI getGUI () {
		
		return ventanaPrincipal;
	}

	public  void enlazarAyuda ( JMenuItem menuItem ,
			Container contentPane ) 
		throws MalformedURLException, HelpSetException {
		
		/** Carga el fichero de ayuda. */
		File fichero = new File( "src/ayuda/help_set.hs" );
		java.net.URL hsURL = fichero.toURI( ).toURL( );
		
		/** Crea el HelpSet */
		/*HelpSet helpset = new HelpSet( getClass( ).getClassLoader( ) ,
				hsURL );*/
		HelpSet helpset = new HelpSet( null ,
				hsURL ); 
		
		HelpSet.Presentation hsp = helpset.getPresentation("MainWin");
		
		HelpBroker help_browser = helpset.createHelpBroker();     
		help_browser.setHelpSetPresentation(hsp);
		
		/** Ayuda al hacer click en el JMenuItem itemAyuda. */
		help_browser.enableHelpOnButton( menuItem, "chap_manualusuario", helpset );

		/** Ayuda al pulsar F1 sobre la ventana principal */
		help_browser.enableHelpKey( contentPane, "chap_manualusuario", helpset);
		
	}

	public  void guardarSesion ( File fichero , Sesion sesion ) 
		throws ParserConfigurationException, TransformerException {
		new SesionXMLParser( ).guardarEnFichero( fichero, sesion );
	}

	public  Sesion leerSesion ( File fichero ) 
		throws XPathExpressionException, ParserConfigurationException, 
			SAXException, IOException {
		Sesion s = new SesionXMLParser( ).leerDeFichero( fichero );
		return s;
	}
	
}
